/*
 *
 * (C) 2004-2005 Denora Team
 * Contact us at info@nomadirc.net
 *
 * Please read COPYING and README for furhter details.
 *
 * Based on the original code of Anope by Anope Team.
 * Based on the original code of Thales by Lucas.
 *
 * $Id: chanstats.c 447 2005-08-04 01:21:52Z trystan $
 *
 */

#include "denora.h"

static void make_stats(User * u, char *receiver, char *msg);
static int countsmileys(char *text);
static int check_db(User * u, Channel * c, char *nick, char *chan);
static char *newsplit(char **rest);
ChannelStats *cslists[1024];

/*************************************************************************/

void insert_cs(ChannelStats * cs)
{
    int csindex = CSHASH(cs->name);

    SET_SEGV_LOCATION();

    cs->prev = NULL;
    cs->next = cslists[csindex];
    if (cs->next)
        cs->next->prev = cs;
    cslists[csindex] = cs;
}

/*************************************************************************/

ChannelStats *find_cs(char *mask)
{
    ChannelStats *cs;

    SET_SEGV_LOCATION();

    if (!mask)
        return NULL;
    for (cs = cslists[CSHASH(mask)]; cs; cs = cs->next) {
        if (stricmp(cs->name, mask) == 0)
            return cs;
    }
    return NULL;
}

/*************************************************************************/

/* Load/save data files. */

void load_cs_db(void)
{
    DenoraDBFile *dbptr = scalloc(1, sizeof(DenoraDBFile));
    ChannelStats *cs = NULL;
    char *key, *value;
    int retval = 0;

    fill_db_ptr(dbptr, 0, CS_VERSION, s_StatServ, ChannelStatsDB);
    SET_SEGV_LOCATION();

    /* let's remove existing temp files here, because we only load dbs on startup */
    remove(dbptr->temp_name);

    /* Open the db, fill the rest of dbptr and allocate memory for key and value */
    if (new_open_db_read(dbptr, &key, &value)) {
        denora_free(dbptr);
        return;                 /* Bang, an error occurred */
    }
    SET_SEGV_LOCATION();

    while (1) {
        /* read a new entry and fill key and value with it -Certus */
        retval = new_read_db_entry(&key, &value, dbptr->fptr);

        if (retval == DB_READ_ERROR) {
            alog(LOG_NORMAL, "WARNING! DB_READ_ERROR in %s",
                 dbptr->filename);
            new_close_db(dbptr->fptr, &key, &value);
            denora_free(dbptr);
            return;
        } else if (retval == DB_EOF_ERROR) {
            alog(LOG_EXTRADEBUG, "debug: %s read successfully",
                 dbptr->filename);
            new_close_db(dbptr->fptr, &key, &value);
            denora_free(dbptr);
            return;
        } else if (retval == DB_READ_BLOCKEND) {        /* DB_READ_BLOCKEND */
            /* a ctcp entry has completely been read. put any checks in here! */
        } else {
            /* DB_READ_SUCCESS */

            if (!*value || !*key)
                continue;

            if (!stricmp(key, "name")) {
                cs = makecs(value);
            } else if (!stricmp(key, "flags")) {
                cs->flags = atoi(value);
            }
        }                       /* else */
    }                           /* while */
}

/*************************************************************************/

ChannelStats *makecs(char *mask)
{
    ChannelStats *cs;

    SET_SEGV_LOCATION();

    cs = scalloc(sizeof(ChannelStats), 1);
    cs->name = sstrdup(mask);
    cs->flags = 0;
    insert_cs(cs);
    return cs;
}

/*************************************************************************/

void save_cs_db(void)
{
    DenoraDBFile *dbptr = scalloc(1, sizeof(DenoraDBFile));
    ChannelStats *cs;
    int i;

    fill_db_ptr(dbptr, 0, CS_VERSION, s_StatServ, ChannelStatsDB);
    SET_SEGV_LOCATION();

    /* time to backup the old db */
    rename(ChannelStatsDB, dbptr->temp_name);

    if (new_open_db_write(dbptr)) {
        rename(dbptr->temp_name, ChannelStatsDB);
        free(dbptr);
        return;                 /* Bang, an error occurred */
    }

    for (i = 0; i < 1024; i++) {
        for (cs = cslists[i]; cs; cs = cs->next) {
            new_write_db_entry("name", dbptr, "%s", cs->name);
            new_write_db_entry("flags", dbptr, "%d", cs->flags);
            new_write_db_endofblock(dbptr);
        }
    }
    SET_SEGV_LOCATION();

    if (dbptr) {
        new_close_db(dbptr->fptr, NULL, NULL);  /* close file */
        remove(dbptr->temp_name);       /* saved successfully, no need to keep the old one */
        free(dbptr);            /* free the db struct */
    }
}

/*************************************************************************/

int del_cs(ChannelStats * cs)
{
    SET_SEGV_LOCATION();

    if (cs) {
        if (cs->prev) {
            cs->prev->next = cs->next;
        } else {
            cslists[CSHASH(cs->name)] = cs->next;
        }
        if (cs->next) {
            cs->next->prev = cs->prev;
        }
        denora_free(cs->name);
        denora_free(cs);
    }

    return 0;
}

/*****************************************************************************************/

void do_cstats(User * u, char *receiver, char *msg)
{
    int i = 1;
    char *command, *buf = NULL, *chan;
    ChannelStats *cs;
#ifdef USE_MYSQL
    MYSQL_RES *mysql_res;
#endif
    int peak = 0;

    SET_SEGV_LOCATION();

#ifndef USE_MYSQL
    /* without sql cstats is pointless */
#else
    if (denora->do_sql) {
        if (!(cs = find_cs(receiver))) {
            /* no error message if you place statserv in the logchan which is the same as
               services logchan it will throw fatal errors */
            return;
        }

        if (cs->flags & CS_FANTASY) {
            buf = sstrdup(msg);
            command = strtok(buf, " ");
            SET_SEGV_LOCATION();
            if (command && !stricmp(command, "!help")) {
                notice_lang(s_StatServ, u, STATS_HELP_CHANNEL_1);
                notice_lang(s_StatServ, u, STATS_HELP_CHANNEL_2);
            } else if (command && !stricmp(command, "!top10")) {        /* top10 of channel */
                chan = rdb_escape(receiver);
                rdb_query
                    ("SELECT * FROM %s WHERE chan=\"%s\" AND type=0 ORDER BY letters DESC LIMIT 10;",
                     UStatsTable, chan);
                denora_free(chan);
                mysql_res = mysql_store_result(mysql);
                if (mysql_num_rows(mysql_res) > 0) {
                    if (cs->flags & CS_NOTICE) {
                        notice_lang(s_StatServ, u, STATS_TOP10_CHANNEL,
                                    receiver);
                    } else {
                        denora_cmd_privmsg(s_StatServ, receiver,
                                           getstring(NULL,
                                                     STATS_TOP10_CHANNEL),
                                           receiver);
                    }
                    SET_SEGV_LOCATION();
                    while ((mysql_row =
                            mysql_fetch_row(mysql_res)) != NULL) {
                        if (cs->flags & CS_NOTICE) {
                            notice_lang(s_StatServ, u, STATS_MESSAGE,
                                        i, mysql_row[0], mysql_row[3],
                                        mysql_row[4], mysql_row[5],
                                        mysql_row[7], mysql_row[6]);
                        } else {
                            denora_cmd_privmsg(s_StatServ, receiver,
                                               getstring(NULL,
                                                         STATS_MESSAGE), i,
                                               mysql_row[0], mysql_row[3],
                                               mysql_row[4], mysql_row[5],
                                               mysql_row[7], mysql_row[6]);
                        }
                        i++;
                    }
                }
                SET_SEGV_LOCATION();
                denora_free(buf);
                mysql_free_result(mysql_res);
                return;
            } else if (command && !stricmp(command, "!peak")) {
                chan = rdb_escape(receiver);
                strtolwr(chan);
                rdb_query
                    ("SELECT maxusers FROM %s WHERE channel=\"%s\";",
                     ChanTable, chan);
                alog(LOG_DEBUG, "!peak: Searching for %s", chan);
                denora_free(chan);
                mysql_res = mysql_store_result(mysql);
                if (mysql_res) {
                    if (mysql_num_rows(mysql_res)) {
                        SET_SEGV_LOCATION();
                        peak = atoi(*mysql_fetch_row(mysql_res));
                        alog(LOG_DEBUG, "!peak: peak number is %d", peak);
                        if (cs->flags & CS_NOTICE) {
                            notice_lang(s_StatServ, u, STATS_CHANNEL_PEAK,
                                        peak);
                        } else {
                            denora_cmd_privmsg(s_StatServ, receiver,
                                               getstring(NULL,
                                                         STATS_CHANNEL_PEAK),
                                               peak);
                        }
                    }
                    SET_SEGV_LOCATION();
                    alog(LOG_DEBUG, "!peak: number of rows found to be 0");
                    mysql_free_result(mysql_res);
                }
                alog(LOG_DEBUG, "!peak: the resource was NULL");
                return;
            } else if (command && !stricmp(command, "!stats")) {        /* top10 of channel */
                if (!u->sgroup) {
                    return;
                }
                chan = rdb_escape(receiver);
                rdb_query
                    ("SELECT * FROM %s WHERE chan=\"%s\" AND type=0 AND uname=\"%s\";",
                     UStatsTable, chan, u->sgroup);
                denora_free(chan);
                mysql_res = mysql_store_result(mysql);
                if (mysql_num_rows(mysql_res) > 0) {
                    SET_SEGV_LOCATION();
                    while ((mysql_row =
                            mysql_fetch_row(mysql_res)) != NULL) {
                        if (cs->flags & CS_NOTICE) {
                            notice_lang(s_StatServ, u, STATS_USER_CHANNEL,
                                        u->nick, receiver);
                            notice_lang(s_StatServ, u, STATS_MESSAGE_ONE,
                                        mysql_row[3], mysql_row[4],
                                        mysql_row[5], mysql_row[7],
                                        mysql_row[6]);
                        } else {
                            denora_cmd_privmsg(s_StatServ, receiver,
                                               getstring(NULL,
                                                         STATS_USER_CHANNEL),
                                               u->nick, receiver);
                            denora_cmd_privmsg(s_StatServ, receiver,
                                               getstring(NULL,
                                                         STATS_MESSAGE_ONE),
                                               mysql_row[3], mysql_row[4],
                                               mysql_row[5], mysql_row[7],
                                               mysql_row[6]);
                        }

                    }
                }
                SET_SEGV_LOCATION();
                denora_free(buf);
                mysql_free_result(mysql_res);
                return;
            } else if (command && !stricmp(command, "!gstats")) {       /* top10 of channel */
                if (!u->sgroup) {
                    return;
                }
                rdb_query
                    ("SELECT * FROM %s WHERE chan=\"global\" AND type=0 AND uname=\"%s\";",
                     UStatsTable, u->sgroup);
                mysql_res = mysql_store_result(mysql);
                if (mysql_num_rows(mysql_res) > 0) {
                    SET_SEGV_LOCATION();
                    while ((mysql_row =
                            mysql_fetch_row(mysql_res)) != NULL) {
                        if (cs->flags & CS_NOTICE) {
                            notice_lang(s_StatServ, u, STATS_USER_NETWORK,
                                        u->nick);
                            notice_lang(s_StatServ, u, STATS_MESSAGE_ONE,
                                        mysql_row[3], mysql_row[4],
                                        mysql_row[5], mysql_row[7],
                                        mysql_row[6]);
                        } else {
                            denora_cmd_privmsg(s_StatServ, receiver,
                                               getstring(NULL,
                                                         STATS_USER_NETWORK),
                                               u->nick);
                            denora_cmd_privmsg(s_StatServ, receiver,
                                               getstring(NULL,
                                                         STATS_MESSAGE_ONE),
                                               mysql_row[3], mysql_row[4],
                                               mysql_row[5], mysql_row[7],
                                               mysql_row[6]);
                        }

                    }
                }
                SET_SEGV_LOCATION();
                denora_free(buf);
                mysql_free_result(mysql_res);
                return;
            } else if (command && !stricmp(command, "!top")) {  /* top 3 of channel */
                SET_SEGV_LOCATION();
                chan = rdb_escape(receiver);
                rdb_query
                    ("SELECT * FROM %s WHERE chan=\"%s\" AND type=0 ORDER BY letters DESC LIMIT 3;",
                     UStatsTable, chan);
                denora_free(chan);
                mysql_res = mysql_store_result(mysql);
                if (mysql_num_rows(mysql_res) > 0) {
                    if (cs->flags & CS_NOTICE) {
                        notice_lang(s_StatServ, u, STATS_TOP_CHANNEL,
                                    receiver);
                    } else {
                        denora_cmd_privmsg(s_StatServ, receiver,
                                           getstring(NULL,
                                                     STATS_TOP_CHANNEL),
                                           receiver);
                    }
                    SET_SEGV_LOCATION();
                    while ((mysql_row =
                            mysql_fetch_row(mysql_res)) != NULL) {
                        if (cs->flags & CS_NOTICE) {
                            notice_lang(s_StatServ, u, STATS_MESSAGE,
                                        i, mysql_row[0], mysql_row[3],
                                        mysql_row[4], mysql_row[5],
                                        mysql_row[7], mysql_row[6]);
                        } else {
                            denora_cmd_privmsg(s_StatServ, receiver,
                                               getstring(NULL,
                                                         STATS_MESSAGE), i,
                                               mysql_row[0], mysql_row[3],
                                               mysql_row[4], mysql_row[5],
                                               mysql_row[7], mysql_row[6]);
                        }
                        i++;
                    }
                }
                SET_SEGV_LOCATION();
                denora_free(buf);
                mysql_free_result(mysql_res);
                return;
            } else if (command && !stricmp(command, "!gtop10")) {       /* top10 of network */
                SET_SEGV_LOCATION();
                rdb_query
                    ("SELECT * FROM %s WHERE chan=\"global\" AND type=0 ORDER BY letters DESC LIMIT 10;",
                     UStatsTable);
                mysql_res = mysql_store_result(mysql);
                if (mysql_num_rows(mysql_res) > 0) {
                    if (cs->flags & CS_NOTICE) {
                        notice_lang(s_StatServ, u, STATS_TOP10_NETWORK,
                                    receiver);
                    } else {
                        denora_cmd_privmsg(s_StatServ, receiver,
                                           getstring(NULL,
                                                     STATS_TOP10_NETWORK),
                                           receiver);
                    }
                    SET_SEGV_LOCATION();
                    while ((mysql_row =
                            mysql_fetch_row(mysql_res)) != NULL) {
                        if (cs->flags & CS_NOTICE) {
                            notice_lang(s_StatServ, u, STATS_MESSAGE,
                                        i, mysql_row[0], mysql_row[3],
                                        mysql_row[4], mysql_row[5],
                                        mysql_row[7], mysql_row[6]);
                        } else {
                            denora_cmd_privmsg(s_StatServ, receiver,
                                               getstring(NULL,
                                                         STATS_MESSAGE), i,
                                               mysql_row[0], mysql_row[3],
                                               mysql_row[4], mysql_row[5],
                                               mysql_row[7], mysql_row[6]);
                        }
                        i++;
                    }
                }
                SET_SEGV_LOCATION();
                denora_free(buf);
                mysql_free_result(mysql_res);
                return;
            } else if (command && !stricmp(command, "!gtop")) { /* top 3 of network */
                SET_SEGV_LOCATION();
                rdb_query
                    ("SELECT * FROM %s WHERE chan=\"global\" AND type=0 ORDER BY letters DESC LIMIT 3;",
                     UStatsTable);
                mysql_res = mysql_store_result(mysql);
                if (mysql_num_rows(mysql_res) > 0) {
                    if (cs->flags & CS_NOTICE) {
                        notice_lang(s_StatServ, u, STATS_TOP_NETWORK,
                                    receiver);
                    } else {
                        denora_cmd_privmsg(s_StatServ, receiver,
                                           getstring(NULL,
                                                     STATS_TOP_NETWORK),
                                           receiver);
                    }
                    SET_SEGV_LOCATION();
                    while ((mysql_row =
                            mysql_fetch_row(mysql_res)) != NULL) {
                        if (cs->flags & CS_NOTICE) {
                            notice_lang(s_StatServ, u, STATS_MESSAGE,
                                        i, mysql_row[0], mysql_row[3],
                                        mysql_row[4], mysql_row[5],
                                        mysql_row[7], mysql_row[6]);
                        } else {
                            denora_cmd_privmsg(s_StatServ, receiver,
                                               getstring(NULL,
                                                         STATS_MESSAGE), i,
                                               mysql_row[0], mysql_row[3],
                                               mysql_row[4], mysql_row[5],
                                               mysql_row[7], mysql_row[6]);
                        }
                        i++;
                    }
                }
                SET_SEGV_LOCATION();
                denora_free(buf);
                mysql_free_result(mysql_res);
                return;
            } else if (command && (command[0] == '!')) {
                char *params = strtok(NULL, "");
                if (params) {
                    send_event(EVENT_FANTASY, 4, command, u->nick,
                               receiver, params);
                } else {
                    send_event(EVENT_FANTASY, 3, command, u->nick,
                               receiver);
                }
            }
        }
        /* BS_FANTASY */
#endif
        SET_SEGV_LOCATION();
        make_stats(u, receiver, msg);
        denora_free(buf);
#ifdef USE_MYSQL
    }
#endif
}

/******************************************************************************************/

static void make_stats(User * u, char *receiver, char *msg)
{
    int letters = 0, words = 1, action = 0, smileys = 0, i = 0;
    char *nick, *chan;
    char *buf;
    Channel *c;
    int hour;

    SET_SEGV_LOCATION();

    nick = rdb_escape(u->nick);
    chan = rdb_escape(receiver);
    c = findchan(receiver);
    check_db(u, c, nick, chan);
    if (u->cstats == 2) {       /* ignore */
        SET_SEGV_LOCATION();
        denora_free(nick);
        denora_free(chan);
        return;
    }
    SET_SEGV_LOCATION();
    buf = normalizeBuffer(msg); /* remove control letters from message */
    letters = strlen(buf);      /* count letters */
    for (i = 0; i < letters; i++) {     /* count words */
        if ((buf[i] == ' ') && (buf[i + 1] != ' '))
            words++;
    }
    smileys = countsmileys(buf);        /* count smileys */
    denora_free(buf);
    /* is a action ? */
    if (strncmp("\01ACTION", msg, 7) == 0) {    /* is a action ? */
        action = 1;
        letters = letters - 9;  /* remove from the counted letters */
        words = words - 1;      /* remove from the counted words */
    }
    words = words - smileys;    /* do not count smileys as words */
    SET_SEGV_LOCATION();
    hour = get_hour();
    /* update user SQL */
    /* update user */
    rdb_query
        ("UPDATE %s SET letters=letters+%i, words=words+%i, line=line+1, "
         "actions=actions+%i, smileys=smileys+%i, lastspoke=%i, time%i=time%i+1 "
         "WHERE (uname=\"%s\" AND (chan=\"global\" OR chan=\"%s\"));",
         UStatsTable, letters, words, action, smileys, time(NULL), hour,
         hour, u->sgroup, chan);
    SET_SEGV_LOCATION();

/* update chan */
    rdb_query
        ("UPDATE %s SET letters=letters+%i, words=words+%i, line=line+1, "
         "actions=actions+%i, smileys=smileys+%i, lastspoke=lastspoke+%i, time%i=time%i+1 WHERE chan=\"%s\";",
         CStatsTable, letters, words, action, smileys, time(NULL), hour,
         hour, chan);
    denora_free(nick);
    denora_free(chan);
}

/*************************************************************************/

void count_kicks(User * kicker, User * kicked, Channel * c)
{
    char *kicker_, *kicked_, *chan_;

#ifdef USE_MYSQL
    if (!c) {
        return;
    }
    if (!find_cs(c->name)) {
        return;
    }

    SET_SEGV_LOCATION();

    chan_ = rdb_escape(c->name);

    if (kicker && !find_exclude(kicker->nick, kicker->server->name)) {
        kicker_ = rdb_escape(kicker->nick);
        check_db(kicker, c, kicker_, chan_);
        if (kicker->cstats != 2) {      /* check for ignore */
            rdb_query
                ("UPDATE %s SET kicks=kicks+1 "
                 "WHERE (uname=\"%s\" AND (chan=\"global\" OR chan=\"%s\"));",
                 UStatsTable, kicker->sgroup, chan_);
            rdb_query
                ("UPDATE %s SET kicks=kicks+1 WHERE chan=\"%s\";",
                 CStatsTable, chan_);
        }
        denora_free(kicker_);
    }

    SET_SEGV_LOCATION();

    if (kicked && !find_exclude(kicked->nick, kicked->server->name)) {
        kicked_ = rdb_escape(kicked->nick);
        check_db(kicked, c, kicked_, chan_);
        if (kicked->cstats != 2) {      /* check for ignore */
            rdb_query
                ("UPDATE %s SET kicked=kicked+1 "
                 "WHERE (uname=\"%s\" AND (chan=\"global\" OR chan=\"%s\"));",
                 UStatsTable, kicked->sgroup, chan_);
        }
        denora_free(kicked_);
    }
    denora_free(chan_);

#endif
}

/*************************************************************************/

void count_topics(User * u, Channel * c)
{
    char *user_, *chan_;

    if (!denora->do_sql) {
        return;
    }

    if (!find_cs(c->name)) {
        return;
    }
    SET_SEGV_LOCATION();

    user_ = rdb_escape(u->nick);
    chan_ = rdb_escape(c->name);

    check_db(u, c, user_, chan_);

    SET_SEGV_LOCATION();

    if (u->cstats != 2) {       /* check for ignore */
        rdb_query
            ("UPDATE %s SET topics=topics+1 "
             "WHERE (uname=\"%s\" AND (chan=\"global\" OR chan=\"%s\"));",
             UStatsTable, u->sgroup, chan_);
        rdb_query
            ("UPDATE %s SET topics=topics+1 WHERE chan=\"%s\";",
             CStatsTable, chan_);
    }
    SET_SEGV_LOCATION();
    denora_free(user_);
    denora_free(chan_);
}

/*************************************************************************/

void count_modes(User * u, Channel * c)
{
    char *user_, *chan_;
    if (!denora->do_sql) {
        return;
    }

    if (!find_cs(c->name)) {
        return;
    }
    SET_SEGV_LOCATION();

    user_ = rdb_escape(u->nick);
    chan_ = rdb_escape(c->name);

    check_db(u, c, user_, chan_);
    SET_SEGV_LOCATION();
    if (u->cstats != 2) {       /* check for ignore */
        rdb_query
            ("UPDATE %s SET modes=modes+1 "
             "WHERE (uname=\"%s\" AND (chan=\"global\" OR chan=\"%s\"));",
             UStatsTable, u->sgroup, chan_);
        rdb_query
            ("UPDATE %s SET modes=modes+1 WHERE chan=\"%s\";", CStatsTable,
             chan_);
    }
    SET_SEGV_LOCATION();
    denora_free(user_);
    denora_free(chan_);
}

/*************************************************************************/
                                     /* rdb_escape nick and chan ! */
static int check_db(User * u, Channel * c, char *nick, char *chan)
{
    int i;
#ifdef USE_MYSQL
    MYSQL_RES *mysql_res;
#endif
    SET_SEGV_LOCATION();
#ifndef USE_MYSQL
    return 0;
#else
    /* get alias from db */
    if (u->cstats == 0) {
        rdb_query("SELECT uname FROM %s WHERE nick=\"%s\";", AliasesTable,
                  nick);
        mysql_res = mysql_store_result(mysql);
        if (mysql_num_rows(mysql_res) > 0) {    /* exists this nick already in the database? */
            mysql_row = mysql_fetch_row(mysql_res);
            denora_free(u->sgroup);
            u->sgroup = rdb_escape(mysql_row[0]);
            u->cstats = 1;
        } else {                /* num_rows = 0 */
            /* create alias and global */
            rdb_query("INSERT INTO %s SET nick=\"%s\", uname=\"%s\";",
                      AliasesTable, nick, nick);
            for (i = 0; i < 4; i++) {
                rdb_query
                    ("INSERT INTO %s SET uname=\"%s\", chan=\"global\", type=%i;",
                     UStatsTable, nick, i);
            }
            u->cstats = 1;
            denora_free(u->sgroup);
            u->sgroup = sstrdup(nick);
        }
        mysql_free_result(mysql_res);
    }

    SET_SEGV_LOCATION();
    /* check for ignore */
    rdb_query
        ("SELECT %s.ignore FROM %s WHERE nick=\"%s\" AND nick=uname AND %s.ignore='Y';",
         AliasesTable, AliasesTable, u->sgroup, AliasesTable);
    mysql_res = mysql_store_result(mysql);
    if (mysql_res) {
        if (mysql_num_rows(mysql_res) > 0) {
            u->cstats = 2;      /* ignore this user */
        }
        mysql_free_result(mysql_res);
    }

    if (c->cstats == 0) {
        SET_SEGV_LOCATION();
        rdb_query("SELECT chan FROM %s WHERE chan=\"%s\" AND type=0;",
                  CStatsTable, chan);
        mysql_res = mysql_store_result(mysql);
        if (mysql_num_rows(mysql_res) > 0) {    /* exists this chan already in the database? */
            c->cstats = 1;
        } else {                /* num_rows = 0 */
            /* create total, daily, weekly, monthly */
            for (i = 0; i < 4; i++) {
                rdb_query("INSERT INTO %s SET chan=\"%s\", type=%i;",
                          CStatsTable, chan, i);
            }
            c->cstats = 1;
        }                       /* num_rows */
        mysql_free_result(mysql_res);
    }                           /* c->cstats */
    SET_SEGV_LOCATION();
    if (u->cstats != 2) {
        rdb_query
            ("SELECT uname FROM %s WHERE uname=\"%s\" AND chan=\"%s\";",
             UStatsTable, u->sgroup, chan);
        mysql_res = mysql_store_result(mysql);
        if (mysql_num_rows(mysql_res) > 0) {
            /* do nothing */
        } else {
            /* create entries for total, daily, weekly, monthly, */
            for (i = 0; i < 4; i++) {
                rdb_query
                    ("INSERT INTO %s SET uname=\"%s\", chan=\"%s\", type=%i;",
                     UStatsTable, u->sgroup, chan, i);
            }
        }                       /* num_rows */
    }
    SET_SEGV_LOCATION();
    mysql_free_result(mysql_res);
    return 1;
#endif
}

/*************************************************************************/

/* transfer all stats from user2 to user1 and delete user2 */

void sumuser(User * u, char *user1, char *user2)
{
    User *u2;
    int i;
#ifdef USE_MYSQL
    MYSQL_RES *mysql_res_chans;
    MYSQL_RES *mysql_res;
    MYSQL_ROW mysql_row_chans;
#endif
    char *user1_, *user2_, *chan_;

    if (!denora->do_sql) {
        /* print an error message? */
        return;
    }

    user1_ = rdb_escape(user1);
    user2_ = rdb_escape(user2);
    SET_SEGV_LOCATION();

    rdb_query
        ("SELECT uname FROM %s WHERE uname = \"%s\" AND uname = nick;",
         AliasesTable, user1_);
#ifdef USE_MYSQL
    mysql_res = mysql_store_result(mysql);
    if (mysql_res) {
        if (mysql_num_rows(mysql_res) == 0) {   /* exists this nick already in the database? */
            mysql_free_result(mysql_res);
            notice_lang(s_StatServ, u, STATS_SUMUSER_NOTFOUND, user1);
            goto end;
        }
        mysql_free_result(mysql_res);
    }
#endif
    SET_SEGV_LOCATION();
    rdb_query
        ("SELECT uname FROM %s WHERE uname = \"%s\" AND uname = nick;",
         AliasesTable, user2_);
#ifdef USE_MYSQL
    mysql_res = mysql_store_result(mysql);
    if (mysql_num_rows(mysql_res) == 0) {       /* exists this nick already in the database? */
        mysql_free_result(mysql_res);
        notice_lang(s_StatServ, u, STATS_SUMUSER_NOTFOUND, user2);
        goto end;
    }
    mysql_free_result(mysql_res);
#else
    goto end;
#endif
    /* get all chans of user2 */
    rdb_query("SELECT chan FROM %s WHERE uname = \"%s\" AND type = 0;",
              UStatsTable, user2_);
#ifdef USE_MYSQL
    mysql_res_chans = mysql_store_result(mysql);
    /* for each chan: */
    /* read total, daily, weekly and  monthly stats and add them to user1 */
    while ((mysql_row_chans = mysql_fetch_row(mysql_res_chans)) != NULL) {
        chan_ = rdb_escape(mysql_row_chans[0]);
        /* read total, daily, weekly and  monthly stats and add them to user1 */
        for (i = 0; i < 4; i++) {
            rdb_query
                ("SELECT * FROM %s WHERE uname = \"%s\" AND chan = \"%s\" AND type = %i;",
                 UStatsTable, user2_, chan_, i);
            mysql_res = mysql_store_result(mysql);
            mysql_row = mysql_fetch_row(mysql_res);
            rdb_query("UPDATE %s SET letters=letters+%s, words=words+%s, line=line+%s, actions=actions+%s, smileys=smileys+%s, kicks=kicks+%s, kicked=kicked+%s, modes=modes+%s, topics=topics+%s, wasted=wasted+%s WHERE uname=\"%s\" AND chan=\"%s\" AND type=%i;", UStatsTable, mysql_row[3],        /* letters */
                      mysql_row[4],     /* words */
                      mysql_row[5],     /* lines */
                      mysql_row[6],     /* actions */
                      mysql_row[7],     /* smileys */
                      mysql_row[8],     /* kicks */
                      mysql_row[9],     /* kicked */
                      mysql_row[10],    /* modes */
                      mysql_row[11],    /* topics */
                      mysql_row[13],    /* wasted */
                      user1_, chan_, i);
            mysql_free_result(mysql_res);
        }
        denora_free(chan_);
    }                           /* while */
    SET_SEGV_LOCATION();

    mysql_free_result(mysql_res_chans);
#endif
    /* delete stats of user2 from ustats */
    rdb_query("DELETE FROM %s WHERE uname = \"%s\";", UStatsTable, user2);
    /* change aliases */
    rdb_query("UPDATE %s SET uname = \"%s\" WHERE nick = \"%s\";",
              AliasesTable, user1_, user2_);
    while ((u2 = finduser_by_sgroup(user2, user2_))) {
        denora_free(u2->sgroup);
        u2->sgroup = sstrdup(user1_);
    }

  end:
    SET_SEGV_LOCATION();
    denora_free(user1_);
    denora_free(user2_);
}

/*************************************************************************/

/* Find a user by nick.  Return NULL if user could not be found. */

User *finduser_by_sgroup(char *nick, char *sgroup)
{
    User *user;

    SET_SEGV_LOCATION();

    if (!sgroup || !*sgroup) {
        alog(LOG_DEBUG, langstr(ALOG_DEBUG_BAD_FINDSGROUP));
        return NULL;
    }

    user = userlist[NICKHASH(nick)];
    while (user && user->sgroup && (stricmp(user->sgroup, sgroup) != 0))
        user = user->next;
    return user;
}

/*************************************************************************/

static int countsmileys(char *text)
{
    char buf[1024], *pbuf, *smiley, *p;
    int ismileys = 0;

    sprintf(buf, "%s", Smiley);
    pbuf = buf;
    SET_SEGV_LOCATION();

    while (strlen(pbuf) > 0) {
        smiley = newsplit(&pbuf);
        p = strstr(text, smiley);
        while (p) {
            ismileys++;
            p += strlen(smiley);
            p = strstr(p, smiley);
        }
    }
    SET_SEGV_LOCATION();
    return ismileys;
}

/*************************************************************************/

static char *newsplit(char **rest)
{
    register char *o, *r;

    SET_SEGV_LOCATION();

    if (!rest)
        return *rest = NULL;
    o = *rest;
    while (*o == ' ')
        o++;
    r = o;
    while (*o && (*o != ' '))
        o++;
    if (*o)
        *o++ = 0;
    *rest = o;
    return r;
}

/*************************************************************************/

int chanstats_daily(const char *name)
{
    if (!denora->do_sql) {
        return MOD_CONT;
    }
    if (!name) {
        return MOD_CONT;
    }
    SET_SEGV_LOCATION();
    rdb_query
        ("UPDATE %s SET letters=0, words=0, line=0, actions=0, smileys=0, "
         "kicks=0, modes=0, topics=0, wasted=0, "
         "time0=0, time1=0, time2=0, time3=0, time4=0, time5=0, time6=0, time7=0, "
         "time8=0, time9=0, time10=0, time11=0, time12=0, time13=0, time14=0, "
         "time15=0, time16=0, time17=0, time18=0, time19=0, time20=0, time21=0, "
         "time22=0, time23=0  WHERE type=1;", UStatsTable);
    rdb_query
        ("UPDATE %s SET letters=0, words=0, line=0, actions=0, smileys=0, "
         "kicks=0, modes=0, topics=0, "
         "time0=0, time1=0, time2=0, time3=0, time4=0, time5=0, time6=0, time7=0, "
         "time8=0, time9=0, time10=0, time11=0, time12=0, time13=0, time14=0, "
         "time15=0, time16=0, time17=0, time18=0, time19=0, time20=0, time21=0, "
         "time22=0, time23=0  WHERE type=1;", CStatsTable);
    return MOD_CONT;
}

/*************************************************************************/

int chanstats_weekly(const char *name)
{
    if (!denora->do_sql) {
        return MOD_CONT;
    }
    if (!name) {
        return MOD_CONT;
    }
    SET_SEGV_LOCATION();
    rdb_query
        ("UPDATE %s SET letters=0, words=0, line=0, actions=0, smileys=0, "
         "kicks=0, modes=0, topics=0, wasted=0, "
         "time0=0, time1=0, time2=0, time3=0, time4=0, time5=0, time6=0, time7=0, "
         "time8=0, time9=0, time10=0, time11=0, time12=0, time13=0, time14=0, "
         "time15=0, time16=0, time17=0, time18=0, time19=0, time20=0, time21=0, "
         "time22=0, time23=0  WHERE type=2;", UStatsTable);
    rdb_query
        ("UPDATE %s SET letters=0, words=0, line=0, actions=0, smileys=0, "
         "kicks=0, modes=0, topics=0, "
         "time0=0, time1=0, time2=0, time3=0, time4=0, time5=0, time6=0, time7=0, "
         "time8=0, time9=0, time10=0, time11=0, time12=0, time13=0, time14=0, "
         "time15=0, time16=0, time17=0, time18=0, time19=0, time20=0, time21=0, "
         "time22=0, time23=0  WHERE type=2;", CStatsTable);
    rdb_query
        ("DELETE %s.* FROM %s WHERE lastspoke < %i;",
         UStatsTable, UStatsTable, (time(NULL) - ClearInActive));
    rdb_query
        ("DELETE %s.* FROM %s LEFT JOIN %s ON %s.uname=%s.uname WHERE %s.uname IS NULL;",
         AliasesTable, AliasesTable, UStatsTable, AliasesTable,
         UStatsTable, UStatsTable);
    return MOD_CONT;
}

/*************************************************************************/

int chanstats_month(const char *name)
{
#ifdef USE_MYSQL
    MYSQL_RES *mysql_res;
#endif
    time_t tbuf;
    ChannelStats *cs;
    char *chan_;

    if (!denora->do_sql) {
        return MOD_CONT;
    }
    if (!name) {
        return MOD_CONT;
    }
    SET_SEGV_LOCATION();
    rdb_query
        ("UPDATE %s SET letters=0, words=0, line=0, actions=0, smileys=0, "
         "kicks=0, modes=0, topics=0, wasted=0, "
         "time0=0, time1=0, time2=0, time3=0, time4=0, time5=0, time6=0, time7=0, "
         "time8=0, time9=0, time10=0, time11=0, time12=0, time13=0, time14=0, "
         "time15=0, time16=0, time17=0, time18=0, time19=0, time20=0, time21=0, "
         "time22=0, time23=0 WHERE type=3;", UStatsTable);
    rdb_query
        ("UPDATE %s SET letters=0, words=0, line=0, actions=0, smileys=0, "
         "kicks=0, modes=0, topics=0, "
         "time0=0, time1=0, time2=0, time3=0, time4=0, time5=0, time6=0, time7=0, "
         "time8=0, time9=0, time10=0, time11=0, time12=0, time13=0, time14=0, "
         "time15=0, time16=0, time17=0, time18=0, time19=0, time20=0, time21=0, "
         "time22=0, time23=0 WHERE type=3;", CStatsTable);

    /* request 109 -  auto-delete channel after not used for 1 month */

    tbuf = (time(NULL) - ClearChanInActive);
    rdb_query
        ("SELECT chan FROM %s WHERE (lastspoke > 0) AND (lastspoke < %i);",
         CStatsTable, tbuf);
#ifdef USE_MYSQL
    mysql_res = mysql_store_result(mysql);
    if (mysql_num_rows(mysql_res) > 0) {
        SET_SEGV_LOCATION();
        while ((mysql_row = mysql_fetch_row(mysql_res)) != NULL) {
            if ((cs = find_cs(mysql_row[0]))) {
                alog(LOG_DEBUG,
                     "chanstats monthly: channel %s is expired, statserv will leave this chan, all stats are deleted");
                del_cs(cs);     /* make statserv part the chan */
                chan_ = rdb_escape(mysql_row[0]);
                rdb_query("DELETE FROM %s WHERE chan=\"%s\"", CStatsTable,
                          chan_);
                rdb_query("DELETE FROM %s WHERE chan=\"%s\"", UStatsTable,
                          chan_);
                denora_free(chan_);
                if (LogChannel) {
                    if (stricmp(LogChannel, mysql_row[0])) {
                        denora_cmd_part(s_StatServ, mysql_row[0],
                                        getstring(NULL,
                                                  STATS_CHANSTATS_PART),
                                        mysql_row[0]);

                    }
                }
            }
        }
        save_cs_db();
    }
#endif
    return MOD_CONT;
}

/*************************************************************************/

int get_hour(void)
{
    time_t now;
    static struct tm *t;
    SET_SEGV_LOCATION();
    now = time(NULL);
    t = localtime(&now);
    return t->tm_hour;
}

/*************************************************************************/
