/*
 *
 * (C) 2004-2005 Denora Team
 * Contact us at info@nomadirc.net
 *
 * Please read COPYING and README for furhter details.
 *
 * Based on the original code of Anope by Anope Team.
 * Based on the original code of Thales by Lucas.
 * 
 * $Id: core.c 442 2005-08-01 19:01:51Z trystan $
 *
 */

#include "denora.h"

/*************************************************************************/

/* need to create a loop to look for the various  */
char *char_encode(char *s1)
{
    int i;
    unsigned char c;
    char buf2[15];
    char buf3[BUFSIZE];
    char buf4[BUFSIZE];
    *buf2 = '\0';
    *buf3 = '\0';
    *buf4 = '\0';

    SET_SEGV_LOCATION();

    if (BadPtr(s1)) {
        return sstrdup("");
    }

    for (i = 0; i <= (int) strlen(s1) - 1; i++) {
        c = s1[i];
        if (c > 127) {
            snprintf(buf2, 15, "&#%ld;", (long int) c);
            if (buf4) {
                snprintf(buf3, BUFSIZE, "%s%s", buf4, buf2);
            } else {
                snprintf(buf3, BUFSIZE, "%s", buf2);
            }
            snprintf(buf4, BUFSIZE, "%s", buf3);
        } else if (c == '&') {
            if (buf4) {
                snprintf(buf3, BUFSIZE, "%s%s", buf4, "&amp;");
            } else {
                snprintf(buf3, BUFSIZE, "%s", "&amp;");
            }
            snprintf(buf4, BUFSIZE, "%s", buf3);
        } else if (c == '<') {
            if (buf4) {
                snprintf(buf3, BUFSIZE, "%s%s", buf4, "&lt;");
            } else {
                snprintf(buf3, BUFSIZE, "%s", "&lt;");
            }
            snprintf(buf4, BUFSIZE, "%s", buf3);
        } else if (c == '>') {
            if (buf4) {
                snprintf(buf3, BUFSIZE, "%s%s", buf4, "&gt;");
            } else {
                snprintf(buf3, BUFSIZE, "%s", "&gt;");
            }
            snprintf(buf4, BUFSIZE, "%s", buf3);
        } else if (c == '"') {
            if (buf4) {
                snprintf(buf3, BUFSIZE, "%s%s", buf4, "&quot;");
            } else {
                snprintf(buf3, BUFSIZE, "%s", "&quot;");
            }
            snprintf(buf4, BUFSIZE, "%s", buf3);
        } else {
            if (buf4) {
                snprintf(buf3, BUFSIZE, "%s%c", buf4, c);
            } else {
                snprintf(buf3, BUFSIZE, "%c", c);
            }
            snprintf(buf4, BUFSIZE, "%s", buf3);
        }
    }

    return sstrdup(buf4);
}

/*************************************************************************/

void ntoa(struct in_addr addr, char *ipaddr, int len)
{
    unsigned char *bytes = (unsigned char *) &addr.s_addr;
    SET_SEGV_LOCATION();
    snprintf(ipaddr, len, "%u.%u.%u.%u", bytes[0], bytes[1], bytes[2],
             bytes[3]);
}

/*************************************************************************/

/**
 * toupper:  Like the ANSI functions, but make sure we return an
 *                   int instead of a (signed) char.
 * @param c Char
 * @return int
 */
int toupper(char c)
{
    SET_SEGV_LOCATION();

    if (islower(c)) {
        return (unsigned char) c - ('a' - 'A');
    } else {
        return (unsigned char) c;
    }
}

/*************************************************************************/

/**
 * tolower:  Like the ANSI functions, but make sure we return an
 *                   int instead of a (signed) char.
 * @param c Char
 * @return int
 */
int tolower(char c)
{
    SET_SEGV_LOCATION();

    if (isupper(c)) {
        return (unsigned char) c + ('a' - 'A');
    } else {
        return (unsigned char) c;
    }
}

/*************************************************************************/

/**
 * strscpy:  Copy at most len-1 characters from a string to a buffer, and
 *           add a null terminator after the last character copied.
 * @param d Buffer to copy into
 * @param s Data to copy int
 * @param len Length of data
 * @return updated buffer
 */
char *strscpy(char *d, const char *s, size_t len)
{
    char *d_orig = d;

    SET_SEGV_LOCATION();

    if (!len) {
        return d;
    }
    while (--len && (*d++ = *s++));
    *d = '\0';
    return d_orig;
}

/*************************************************************************/

/**
 * stristr:  Search case-insensitively for string s2 within string s1,
 *           returning the first occurrence of s2 or NULL if s2 was not
 *           found.
 * @param s1 String 1
 * @param s2 String 2
 * @return first occurrence of s2
 */
char *stristr(char *s1, char *s2)
{
    register char *s = s1, *d = s2;

    SET_SEGV_LOCATION();

    while (*s1) {
        if (tolower(*s1) == tolower(*d)) {
            s1++;
            d++;
            if (*d == 0)
                return s;
        } else {
            s = ++s1;
            d = s2;
        }
    }
    return NULL;
}

/*************************************************************************/

/**
 * strnrepl:  Replace occurrences of `old' with `new' in string `s'.  Stop
 *            replacing if a replacement would cause the string to exceed
 *            `size' bytes (including the null terminator).  Return the
 *            string.
 * @param s String
 * @param size size of s
 * @param old character to replace
 * @param new character to replace with
 * @return updated s
 */
char *strnrepl(char *s, int32 size, const char *old, const char *new)
{
    char *ptr = s;
    int32 left = strlen(s);
    int32 avail = size - (left + 1);
    int32 oldlen = strlen(old);
    int32 newlen = strlen(new);
    int32 diff = newlen - oldlen;

    SET_SEGV_LOCATION();

    while (left >= oldlen) {
        if (strncmp(ptr, old, oldlen) != 0) {
            left--;
            ptr++;
            continue;
        }
        if (diff > avail)
            break;
        if (diff != 0)
            memmove(ptr + oldlen + diff, ptr + oldlen, left + 1);
        strncpy(ptr, new, newlen);
        ptr += newlen;
        left -= oldlen;
    }
    return s;
}

/*************************************************************************/

/**
 * merge_args:  Take an argument count and argument vector and merge them
 *              into a single string in which each argument is separated by
 *              a space.
 * @param int Number of Args
 * @param argv Array
 * @return string of the merged array
 */
char *merge_args(int argc, char **argv)
{
    int i;
    static char s[4096];
    char *t;

    SET_SEGV_LOCATION();

    t = s;
    for (i = 0; i < argc; i++)
        t += snprintf(t, sizeof(s) - (t - s), "%s%s", *argv++,
                      (i < argc - 1) ? " " : "");
    return s;
}

/*************************************************************************/

/**
 * print_args:  Take an argument count and argument vector and merge them
 *              into a single string in which each argument is separated by
 *              a space.
 * @param int Number of Args
 * @param argv Array
 * @return string of the merged array
 */
void protocol_debug(char *source, int argc, char **argv)
{
    int i;
    SET_SEGV_LOCATION();

    if (source) {
        alog(LOG_PROTOCOL, "Source %s", source);
    }

    for (i = 0; i < argc; i++) {
        alog(LOG_PROTOCOL, "ac[%d] = %s", i, argv[i]);
    }
    return;
}

/*************************************************************************/

/**
 * do_match_wild:  Attempt to match a string to a pattern which might contain
 *              '*' or '?' wildcards.  Return 1 if the string matches the
 *              pattern, 0 if not.
 * @param pattern To be matched
 * @param str String in which the pattern is to be matched
 * @param docase Case In/Senstive
 * @return 1 if the string matches the pattern, 0 if not.
 */
static int do_match_wild(const char *pattern, const char *str, int docase)
{
    char c;
    const char *s;

    SET_SEGV_LOCATION();

    if (!str || !*str || !pattern || !*pattern) {
        return 0;
    }

    /* This WILL eventually terminate: either by *pattern == 0, or by a
     * trailing '*'. */

    for (;;) {
        switch (c = *pattern++) {
        case 0:
            if (!*str)
                return 1;
            return 0;
        case '?':
            if (!*str)
                return 0;
            str++;
            break;
        case '*':
            if (!*pattern)
                return 1;       /* trailing '*' matches everything else */
            s = str;
            while (*s) {
                if ((docase ? (*s == *pattern)
                     : (tolower(*s) == tolower(*pattern)))
                    && do_match_wild(pattern, s, docase))
                    return 1;
                s++;
            }
            break;
        default:
            if (docase ? (*str++ != c) : (tolower(*str++) != tolower(c)))
                return 0;
            break;
        }                       /* switch */
    }
}

/*************************************************************************/

/**
 * match_wild:  Case Senstive wild card search
 * @param pattern To be matched
 * @param str String in which the pattern is to be matched
 * @return 1 if the string matches the pattern, 0 if not.
 */
int match_wild(const char *pattern, const char *str)
{
    SET_SEGV_LOCATION();
    return do_match_wild(pattern, str, 1);
}

/*************************************************************************/

/**
 * match_wild:  Case Insenstive wild card search
 * @param pattern To be matched
 * @param str String in which the pattern is to be matched
 * @return 1 if the string matches the pattern, 0 if not.
 */
int match_wild_nocase(const char *pattern, const char *str)
{
    SET_SEGV_LOCATION();
    return do_match_wild(pattern, str, 0);
}

/*************************************************************************/

int match_usermask(const char *mask, User * user)
{
    char *mask2 = sstrdup(mask);
    char *nick, *username, *host;
    int result;

    if (strchr(mask2, '!')) {
        nick = strtok(mask2, "!");
        username = strtok(NULL, "@");
    } else {
        nick = NULL;
        username = strtok(mask2, "@");
    }
    host = strtok(NULL, "");
    if (!username || !host) {
        denora_free(mask2);
        return 0;
    }

    if (nick) {
        result = match_wild_nocase(nick, user->nick)
            && match_wild_nocase(username, user->username)
            && (match_wild_nocase(host, user->host)
                || match_wild_nocase(host, user->vhost));
    } else {
        result = match_wild_nocase(username, user->username)
            && (match_wild_nocase(host, user->host)
                || match_wild_nocase(host, user->vhost));
    }

    denora_free(mask2);
    return result;
}

/*************************************************************************/

/**
 * dotime:  Return the number of seconds corresponding to the given time
 *          string.  If the given string does not represent a valid time,
 *          return -1.
 *
 *          A time string is either a plain integer (representing a number
 *          of seconds), or an integer followed by one of these characters:
 *          "s" (seconds), "m" (minutes), "h" (hours), or "d" (days).
 * @param s String to convert
 * @return int
 */
int dotime(const char *s)
{
    int amount;

    if (!*s) {
        return -1;
    }

    SET_SEGV_LOCATION();

    amount = strtol(s, (char **) &s, 10);
    if (*s) {
        switch (*s) {
        case 's':
            return amount;
        case 'm':
            return amount * 60;
        case 'h':
            return amount * 3600;
        case 'd':
            return amount * 86400;
        default:
            return -2;
        }
    } else {
        return amount;
    }
}

/*************************************************************************/

/**
 * Get the token
 * @param str String to search in
 * @param dilim Character to search for
 * @param token_number the token number
 * @return token
 */
char *myStrGetToken(const char *str, const char dilim, int token_number)
{
    int len, idx, counter = 0, start_pos = 0;
    char *substring = NULL;
    if (!str) {
        return NULL;
    }
    SET_SEGV_LOCATION();

    len = strlen(str);
    for (idx = 0; idx <= len; idx++) {
        if ((str[idx] == dilim) || (idx == len)) {
            if (counter == token_number) {
                substring = myStrSubString(str, start_pos, idx);
                counter++;
            } else {
                start_pos = idx + 1;
                counter++;
            }
        }
    }
    return substring;
}

/*************************************************************************/

/**
 * Get the token only
 * @param str String to search in
 * @param dilim Character to search for
 * @param token_number the token number
 * @return token
 */
char *myStrGetOnlyToken(const char *str, const char dilim,
                        int token_number)
{
    int len, idx, counter = 0, start_pos = 0;
    char *substring = NULL;
    if (!str) {
        return NULL;
    }
    SET_SEGV_LOCATION();

    len = strlen(str);
    for (idx = 0; idx <= len; idx++) {
        if (str[idx] == dilim) {
            if (counter == token_number) {
                if (str[idx] == '\r')
                    substring = myStrSubString(str, start_pos, idx - 1);
                else
                    substring = myStrSubString(str, start_pos, idx);
                counter++;
            } else {
                start_pos = idx + 1;
                counter++;
            }
        }
    }
    return substring;
}

/*************************************************************************/

/**
 * Get the Remaining tokens
 * @param str String to search in
 * @param dilim Character to search for
 * @param token_number the token number
 * @return token
 */
char *myStrGetTokenRemainder(const char *str, const char dilim,
                             int token_number)
{
    int len, idx, counter = 0, start_pos = 0;
    char *substring = NULL;
    if (!str) {
        return NULL;
    }
    len = strlen(str);

    SET_SEGV_LOCATION();

    for (idx = 0; idx <= len; idx++) {
        if ((str[idx] == dilim) || (idx == len)) {
            if (counter == token_number) {
                substring = myStrSubString(str, start_pos, len);
                counter++;
            } else {
                start_pos = idx + 1;
                counter++;
            }
        }
    }
    return substring;
}

/*************************************************************************/

/**
 * Get the string between point A and point B
 * @param str String to search in
 * @param start Point A
 * @param end Point B
 * @return the string in between
 */
char *myStrSubString(const char *src, int start, int end)
{
    char *substring = NULL;
    int len, idx;
    if (!src) {
        return NULL;
    }
    SET_SEGV_LOCATION();

    len = strlen(src);
    if (((start >= 0) && (end <= len)) && (end > start)) {
        substring = (char *) malloc(sizeof(char) * ((end - start) + 1));
        for (idx = 0; idx <= end - start; idx++) {
            substring[idx] = src[start + idx];
        }
        substring[end - start] = '\0';
    }
    return substring;
}

/*************************************************************************/

/**
 * Clean up the buffer for extra spaces
 * @param str to clean up
 * @return void
 */
void doCleanBuffer(char *str)
{
    char *in, *out;
    char ch;

    if (!str) {
        return;
    }

    SET_SEGV_LOCATION();

    in = str;
    out = str;

    while (issp(ch = *in++));
    if (ch != '\0')
        for (;;) {
            *out++ = ch;
            ch = *in++;
            if (ch == '\0')
                break;
            if (!issp(ch))
                continue;
            while (issp(ch = *in++));
            if (ch == '\0')
                break;
            *out++ = ' ';
        }
    *out = ch;                  /* == '\0' */
}

/*************************************************************************/

/**
 * Is the given nick a network service
 * @param nick to check
 * @param int Check if botserv bots
 * @return int
 */
int nickIsServices(char *nick)
{
    User *u;
    int found = 0;

    SET_SEGV_LOCATION();

    if (!nick) {
        return found;
    }

    if (UseTS6 && ircd->ts6) {
        u = find_byuid(nick);
        if (!u) {
            u = finduser(nick);
        }
    } else if (ircd->p10) {
        u = find_byuid(nick);
    } else {
        u = finduser(nick);
    }

    if (!u) {
        return found;
    }

    if (u->isservice) {
        found++;
    }
    return found;
}

/*************************************************************************/

/**
 * Determine if we need to send the TOKEN
 * @param token1 
 * @param token2
 * @return token to send
 */
const char *send_token(const char *token1, const char *token2)
{
    SET_SEGV_LOCATION();

    if (UseTokens && ircd->token && ircdcap->token) {
        return token2;
    } else {
        return token1;
    }
}

/*************************************************************************/

/**
 * Number of tokens in a string
 * @param str String
 * @param dilim Dilimiter
 * @return number of tokens
 */
int myNumToken(const char *str, const char dilim)
{
    int len, idx, counter = 0, start_pos = 0;
    if (!str) {
        return 0;
    }
    SET_SEGV_LOCATION();

    len = strlen(str);
    for (idx = 0; idx <= len; idx++) {
        if ((str[idx] == dilim) || (idx == len)) {
            start_pos = idx + 1;
            counter++;
        }
    }
    return counter;
}

/*************************************************************************/

/**
 * Resolve a host to an IP
 * @param host to convert
 * @return ip address
 */
char *host_resolve(char *host)
{
    struct hostent *hentp = NULL;
    uint32 ip = INADDR_NONE;
    char ipbuf[16];
    char *ipreturn;
    struct in_addr addr;

    ipreturn = NULL;

    SET_SEGV_LOCATION();

#if HAVE_GETHOSTBYNAME
    hentp = gethostbyname(host);

    if (hentp) {
        memcpy(&ip, hentp->h_addr, sizeof(hentp->h_length));
        addr.s_addr = ip;
        ntoa(addr, ipbuf, sizeof(ipbuf));
        ipreturn = sstrdup(ipbuf);
        alog(LOG_DEBUG, langstr(ALOG_DEBUG_RESOLVED_HOST), host, ipbuf);
        return ipreturn;
    } else {
#endif
        return ipreturn;
#if HAVE_GETHOSTBYNAME
    }
#endif
}

/*************************************************************************/
/**
 * Normalize buffer stripping control characters and colors
 * @param A string to be parsed for control and color codes
 * @return A string stripped of control and color codes
 */
char *normalizeBuffer(char *buf)
{
    char *newbuf;
    int i, len, j = 0;

    SET_SEGV_LOCATION();

    len = strlen(buf);
    newbuf = (char *) malloc(sizeof(char) * len + 1);

    for (i = 0; i < len; i++) {
        switch (buf[i]) {
            /* ctrl char */
        case 1:
            break;
            /* Bold ctrl char */
        case 2:
            break;
            /* Color ctrl char */
        case 3:
            /* If the next character is a digit, its also removed */
            if (isdigit(buf[i + 1])) {
                i++;

                /* not the best way to remove colors
                 * which are two digit but no worse then
                 * how the Unreal does with +S - TSL
                 */
                if (isdigit(buf[i + 1])) {
                    i++;
                }

                /* Check for background color code
                 * and remove it as well
                 */
                if (buf[i + 1] == ',') {
                    i++;

                    if (isdigit(buf[i + 1])) {
                        i++;
                    }
                    /* not the best way to remove colors
                     * which are two digit but no worse then
                     * how the Unreal does with +S - TSL
                     */
                    if (isdigit(buf[i + 1])) {
                        i++;
                    }
                }
            }

            break;
            /* Reverse ctrl char */
        case 22:
            break;
            /* Underline ctrl char */
        case 31:
            break;
            /* A valid char gets copied into the new buffer */
        default:
            newbuf[j] = buf[i];
            j++;
        }
    }

    /* Terminate the string */
    newbuf[j] = 0;

    return (newbuf);
}

/*************************************************************************/

/* split_buf:  Split a buffer into arguments and store the arguments in an
 *             argument vector pointed to by argv (which will be malloc'd
 *             as necessary); return the argument count.  If colon_special
 *             is non-zero, then treat a parameter with a leading ':' as
 *             the last parameter of the line, per the IRC RFC.  Destroys
 *             the buffer by side effect.
 */

int split_buf(char *buf, char ***argv)
{
    int argvsize = 8;
    int argc;
    char *s;

    SET_SEGV_LOCATION();

    *argv = scalloc(sizeof(char *) * argvsize, 1);
    argc = 0;
    while (*buf) {
        if (argc == argvsize) {
            argvsize += 8;
            *argv = srealloc(*argv, sizeof(char *) * argvsize);
        }
        if (*buf == ':') {
            (*argv)[argc++] = buf + 1;
            buf = (char *) "";
        } else {
            s = strpbrk(buf, " ");
            if (s) {
                *s++ = 0;
                while (*s == ' ')
                    s++;
            } else {
                s = buf + strlen(buf);
            }
            (*argv)[argc++] = buf;
            buf = s;
        }
    }
    return argc;
}

/* smalloc, scalloc, srealloc, sstrdup:
 *	Versions of the memory allocation functions which will cause the
 *	program to terminate with an "Out of memory" error if the memory
 *	cannot be allocated.  (Hence, the return value from these functions
 *	is never NULL.)
 */

/*************************************************************************/

/**
 * malloc, replacement so we can trap for "out of memory"
 * @param size to allocate
 * @return void
 */
void *smalloc(long size)
{
    void *buf;

    SET_SEGV_LOCATION();

    if (!size) {
        size = 1;
    }
    buf = malloc(size);
    if (!buf)
#if !defined(LINUX20)
#ifndef _WIN32
        raise(SIGUSR1);
#else
        abort();
#endif
#else
        abort();
#endif
    return buf;
}

/*************************************************************************/

/**
 * calloc, replacement so we can trap for "out of memory"
 * @param elsize to allocate
 * @param els size of members
 * @return void
 */
void *scalloc(long elsize, long els)
{
    void *buf;

    SET_SEGV_LOCATION();

    if (!elsize || !els) {
        elsize = els = 1;
    }
    buf = calloc(elsize, els);
    if (!buf)
#if !defined(LINUX20)
#ifndef _WIN32
        raise(SIGUSR1);
#else
        abort();
#endif
#else
        abort();
#endif
    return buf;
}

/*************************************************************************/

/**
 * realloc, replacement so we can trap for "out of memory"
 * @param oldptr Old Pointer
 * @param newsize Size of new pointer
 * @return void
 */
void *srealloc(void *oldptr, long newsize)
{
    void *buf;

    SET_SEGV_LOCATION();

    if (!newsize) {
        newsize = 1;
    }
    buf = realloc(oldptr, newsize);
    if (!buf)
#if !defined(LINUX20)
#ifndef _WIN32
        raise(SIGUSR1);
#else
        abort();
#endif
#else
        abort();
#endif
    return buf;
}

/*************************************************************************/

/**
 * strdup, replacement so we can trap for "out of memory"
 * @param oldptr Old Pointer
 * @param newsize Size of new pointer
 * @return void
 */
char *sstrdup(const char *src)
{
    char *ret = NULL;

    if (src) {
#ifdef __STRICT_ANSI__
        if ((ret = (char *) malloc(strlen(src) + 1))) {
            strcpy(ret, src);
        }
#else
#if !HAVE_STRDUP
        if ((ret = (char *) malloc(strlen(src) + 1))) {
            strcpy(ret, src);
        }
#else
        ret = strdup(src);
#endif
#endif
        if (!ret)
#ifndef _WIN32
            raise(SIGUSR1);
#else
            abort();
#endif
    } else {
        alog(LOG_DEBUG, langstr(ALOG_DEBUG_BAD_SSTRDUP));
        if (denora->debug) {
            do_backtrace(0);
        }
    }
    return ret;
}

/*************************************************************************/

/**
 * strdup, replacement so we can trap for "out of memory"
 * @param oldptr Old Pointer
 * @param newsize Size of new pointer
 * @return void
 */
char *sstrndup(const char *src, size_t n)
{
    char *ret = NULL;

    SET_SEGV_LOCATION();

    if (src) {
        if ((ret = (char *) malloc(strlen(src) + 1))) {
            strncpy(ret, src, n);
        }
        if (!ret)
#ifndef _WIN32
            raise(SIGUSR1);
#else
            abort();
#endif
    }

    return ret;
}

/*************************************************************************/

void strtolwr(char *ch)
{
    SET_SEGV_LOCATION();

    while (*ch) {
        *ch = tolower(*ch);
        ch++;
    }
}

/*************************************************************************/

/**
 * Change an unsigned string to a signed string, overwriting the original
 * string.
 * @param input string
 * @return output string, same as input string.
 */
char *str_signed(unsigned char *str)
{
    char *nstr;

    SET_SEGV_LOCATION();

    nstr = (char *) str;
    while (*str) {
        *nstr = (char) *str;
        str++;
        nstr++;
    }

    return nstr;
}

/*************************************************************************/

char *militime_float(char *start)
{

    static char timebuf[18];
#if HAVE_GETTIMEOFDAY
    struct timeval tv;
    char *p;
    gettimeofday(&tv, NULL);
    if (start) {
        if ((p = strchr(start, '.'))) {
            p++;
            sprintf(timebuf, "%ld",
                    (tv.tv_sec - atoi(start)) * 1000 + (tv.tv_usec -
                                                        atoi(p)) / 1000);
        } else
            strcpy(timebuf, "0");
    } else
        sprintf(timebuf, "%ld.%ld", tv.tv_sec, tv.tv_usec);
    return timebuf;
#else
    sprintf(timebuf, "%ld", (long int) time(NULL));
    return sstrdup(timebuf);
#endif
}

/*************************************************************************/

void post_config_check()
{
    int error = 0;
    if (BadPtr(ServerName)) {
        error++;
    }
    if (BadPtr(ServerDesc)) {
        error++;
    }
    if (BadPtr(ServiceUser)) {
        error++;
    }

    if (!ReadTimeout) {
        alog(LOG_ERROR, langstring(CONFIG_TIMEOUT_READ_ERROR));
        if (!denora->nofork && isatty(2)) {
            fprintf(stderr, langstring(CONFIG_TIMEOUT_READ_ERROR));
        }
        error++;
    }
    if (!WarningTimeout) {
        alog(LOG_ERROR, langstring(CONFIG_TIMEOUT_WARNING_ERROR));
        if (!denora->nofork && isatty(2)) {
            fprintf(stderr, langstring(CONFIG_TIMEOUT_WARNING_ERROR));
        }
        error++;
    }
    if (!UpdateTimeout) {
        alog(LOG_ERROR, langstring(CONFIG_TIMEOUT_UPDATE_ERROR));
        if (!denora->nofork && isatty(2)) {
            fprintf(stderr, langstring(CONFIG_TIMEOUT_UPDATE_ERROR));
        }
        error++;
    }
    if (!PingFrequency) {
        alog(LOG_ERROR, langstring(CONFIG_TIMEOUT_PING_ERROR));
        if (!denora->nofork && isatty(2)) {
            fprintf(stderr, langstring(CONFIG_TIMEOUT_PING_ERROR));
        }
        error++;
    }
    if (!UptimeTimeout) {
        alog(LOG_ERROR, langstring(CONFIG_TIMEOUT_UPTIME_ERROR));
        if (!denora->nofork && isatty(2)) {
            fprintf(stderr, langstring(CONFIG_TIMEOUT_UPTIME_ERROR));
        }
        error++;
    }
    if (!HTMLTimeout) {
        alog(LOG_ERROR, langstring(CONFIG_TIMEOUT_HTML_ERROR));
        if (!denora->nofork && isatty(2)) {
            fprintf(stderr, langstring(CONFIG_TIMEOUT_HTML_ERROR));
        }
        error++;
    }
    if (!SQLPingFreq) {
        alog(LOG_ERROR, langstring(CONFIG_TIMEOUT_SQL_ERROR));
        if (!denora->nofork && isatty(2)) {
            fprintf(stderr, langstring(CONFIG_TIMEOUT_SQL_ERROR));
        }
        error++;
    }
    if (BadPtr(s_StatServ)) {
        alog(LOG_ERROR, langstring(CONFIG_SS_NAME_ERROR));
        if (!denora->nofork && isatty(2)) {
            fprintf(stderr, langstring(CONFIG_SS_NAME_ERROR));
        }
        error++;
    }
    if (BadPtr(desc_StatServ)) {
        alog(LOG_ERROR, langstring(CONFIG_SS_DESC_ERROR));
        if (!denora->nofork && isatty(2)) {
            fprintf(stderr, langstring(CONFIG_SS_DESC_ERROR));
        }
        error++;
    }
    if (s_StatServ_alias && BadPtr(desc_StatServ_alias)) {
        alog(LOG_ERROR, langstring(CONFIG_SS_ALIASDESC_ERROR));
        if (!denora->nofork && isatty(2)) {
            fprintf(stderr, langstring(CONFIG_SS_ALIASDESC_ERROR));
        }
        error++;
    }
    if (s_StatServ_alias) {
        if (BadPtr(s_StatServ_alias)) {
            alog(LOG_ERROR, langstring(CONFIG_SS_ALIASDESC_ERROR));
            if (!denora->nofork && isatty(2)) {
                fprintf(stderr, langstring(CONFIG_SS_ALIASDESC_ERROR));
            }
            error++;
        }
    }


    if (!UserTable) {
        alog(LOG_ERROR, langstring(CONFIG_TABLE_USER_ERROR));
        if (!denora->nofork && isatty(2)) {
            fprintf(stderr, langstring(CONFIG_TABLE_USER_ERROR));
        }
        error++;
    }
    if (!ChanBansTable) {
        alog(LOG_ERROR, langstring(CONFIG_TABLE_CHANBANS_ERROR));
        if (!denora->nofork && isatty(2)) {
            fprintf(stderr, langstring(CONFIG_TABLE_CHANBANS_ERROR));
        }
        error++;
    }
    if (!ChanExceptTable) {
        alog(LOG_ERROR, langstring(CONFIG_TABLE_CHANEXCEPT_ERROR));
        if (!denora->nofork && isatty(2)) {
            fprintf(stderr, langstring(CONFIG_TABLE_CHANEXCEPT_ERROR));
        }
        error++;
    }
    if (!IsOnTable) {
        alog(LOG_ERROR, langstring(CONFIG_TABLE_ISON_ERROR));
        if (!denora->nofork && isatty(2)) {
            fprintf(stderr, langstring(CONFIG_TABLE_ISON_ERROR));
        }
        error++;
    }
    if (!ServerTable) {
        alog(LOG_ERROR, langstring(CONFIG_TABLE_SERVER_ERROR));
        if (!denora->nofork && isatty(2)) {
            fprintf(stderr, langstring(CONFIG_TABLE_SERVER_ERROR));
        }
        error++;
    }
    if (!GlineTable) {
        alog(LOG_ERROR, langstring(CONFIG_TABLE_GLINE_ERROR));
        if (!denora->nofork && isatty(2)) {
            fprintf(stderr, langstring(CONFIG_TABLE_GLINE_ERROR));
        }
        error++;
    }
    if (!ChanTable) {
        alog(LOG_ERROR, "Lacking chan definition in tables block");
        if (!denora->nofork && isatty(2)) {
            fprintf(stderr, "Lacking chan definition in tables block");
        }
        error++;
    }
    if (!MaxValueTable) {
        alog(LOG_ERROR, "Lacking maxvalue definition in tables block");
        if (!denora->nofork && isatty(2)) {
            fprintf(stderr, "Lacking maxvalue definition in tables block");
        }
        error++;
    }
    if (!ChanInviteTable) {
        ChanInviteTable = sstrdup("chaninvites");
    }
    if (!TLDTable) {
        alog(LOG_ERROR, "Lacking tld definition in tables block");
        if (!denora->nofork && isatty(2)) {
            fprintf(stderr, "Lacking tld definition in tables block");
        }
        error++;
    }
    if (!CTCPTable) {
        alog(LOG_ERROR, "Lacking ctcp definition in tables block");
        if (!denora->nofork && isatty(2)) {
            fprintf(stderr, "Lacking ctcp definition in tables block");
        }
        error++;
    }
    if (!SglineTable) {
        SglineTable = sstrdup("sgline");
    }
    if (!SqlineTable) {
        SqlineTable = sstrdup("sqline");
    }

    if (error) {
        alog(LOG_ERROR,
             "Numerious configuration options are incorrectly defined exiting");
        if (!denora->nofork && isatty(2)) {
            fprintf(stderr,
                    "Numerious configuration options are incorrectly defined exiting");
        }
        exit(1);
    }
}

/*************************************************************************/

/* Does the given user have Stats admin privileges? */

int is_stats_admin(User * u)
{
    SET_SEGV_LOCATION();
    if ((is_oper(u)) && (u->admin == 1)) {
        return 1;
    } else {
        return 0;
    }
}
