/*
 *
 * (C) 2004-2005 Denora Team
 * Contact us at info@nomadirc.net
 *
 * Please read COPYING and README for furhter details.
 *
 * Based on the original code of Anope by Anope Team.
 * Based on the original code of Thales by Lucas.
 * 
 * $Id: ctcp.c 387 2005-07-04 16:49:04Z trystan $
 *
 */

#include "denora.h"

CTCPVerStats *ctcplists[1024];
static CTCPVerStats *current;
static int next_index;

/*************************************************************************/

void insert_ctcp(CTCPVerStats * c)
{
    int ctcpindex = CTCPHASH(c->version);

    SET_SEGV_LOCATION();

    c->prev = NULL;
    c->next = ctcplists[ctcpindex];
    if (c->next)
        c->next->prev = c;
    ctcplists[ctcpindex] = c;
}

/*************************************************************************/

CTCPVerStats *find_ctcpver(char *mask)
{
    CTCPVerStats *c;

    SET_SEGV_LOCATION();

    if (!mask)
        return NULL;
    for (c = ctcplists[CTCPHASH(mask)]; c; c = c->next) {
        if (stricmp(c->version, mask) == 0)
            return c;
    }
    return NULL;
}

/*************************************************************************/

/* Load/save data files. */
void load_ctcp_db(void)
{
    DenoraDBFile *dbptr = scalloc(1, sizeof(DenoraDBFile));
    CTCPVerStats *ct = NULL;
    char *key, *value;
    int retval = 0;
    char *version;

    fill_db_ptr(dbptr, 0, CTCP_VERSION, s_StatServ, ctcpDB);
    SET_SEGV_LOCATION();

    /* let's remove existing temp files here, because we only load dbs on startup */
    remove(dbptr->temp_name);

    /* Open the db, fill the rest of dbptr and allocate memory for key and value */
    if (new_open_db_read(dbptr, &key, &value)) {
        free(dbptr);
        return;                 /* Bang, an error occurred */
    }
    SET_SEGV_LOCATION();

    while (1) {
        /* read a new entry and fill key and value with it -Certus */
        retval = new_read_db_entry(&key, &value, dbptr->fptr);

        if (retval == DB_READ_ERROR) {
            alog(LOG_NORMAL, langstr(ALOG_DB_ERROR), dbptr->filename);
            new_close_db(dbptr->fptr, &key, &value);
            free(dbptr);
            return;
        } else if (retval == DB_EOF_ERROR) {
            alog(LOG_EXTRADEBUG, langstr(ALOG_DEBUG_DB_OK),
                 dbptr->filename);
            new_close_db(dbptr->fptr, &key, &value);
            free(dbptr);
            return;
        } else if (retval == DB_READ_BLOCKEND) {        /* DB_READ_BLOCKEND */
            /* a ctcp entry has completely been read. put any checks in here! */
        } else {
            /* DB_READ_SUCCESS */

            if (!*value || !*key) {
                continue;
            }

            if (!stricmp(key, "version")) {
                version = sstrdup(value);
                strnrepl(version, BUFSIZE, "", ":");
                ct = makectcp(version);
                denora_free(version);
            } else if (!stricmp(key, "count")) {
                ct->overall = atoi(value);
            }
        }                       /* else */
    }                           /* while */
}

/*************************************************************************/

CTCPVerStats *makectcp(char *mask)
{
    CTCPVerStats *c;

    c = scalloc(sizeof(CTCPVerStats), 1);
    SET_SEGV_LOCATION();

    c->version = sstrdup(mask);
    insert_ctcp(c);
    return c;
}

/*************************************************************************/

void save_ctcp_db(void)
{
    DenoraDBFile *dbptr = scalloc(1, sizeof(DenoraDBFile));
    CTCPVerStats *c;
    int i;
    char *version;

    fill_db_ptr(dbptr, 0, CTCP_VERSION, s_StatServ, ctcpDB);
    SET_SEGV_LOCATION();

    /* time to backup the old db */
    rename(ctcpDB, dbptr->temp_name);

    if (new_open_db_write(dbptr)) {
        rename(dbptr->temp_name, ctcpDB);
        free(dbptr);
        return;                 /* Bang, an error occurred */
    }
    SET_SEGV_LOCATION();

    for (i = 0; i < 1024; i++) {
        for (c = ctcplists[i]; c; c = c->next) {
            version = sstrdup(c->version);
            strnrepl(version, BUFSIZE, ":", "");
            new_write_db_entry("version", dbptr, "%s", version);
            denora_free(version);
            new_write_db_entry("count", dbptr, "%d", c->overall);
            new_write_db_endofblock(dbptr);
        }
    }

    if (dbptr) {
        new_close_db(dbptr->fptr, NULL, NULL);  /* close file */
        SET_SEGV_LOCATION();
        remove(dbptr->temp_name);       /* saved successfully, no need to keep the old one */
        free(dbptr);            /* free the db struct */
    }
}

/*************************************************************************/

int del_ctcpver(CTCPVerStats * c)
{
    SET_SEGV_LOCATION();

    if (c) {
        if (c->prev) {
            c->prev->next = c->next;
        } else {
            ctcplists[CTCPHASH(c->version)] = c->next;
        }
        if (c->next) {
            c->next->prev = c->prev;
        }
        if (c->version) {
            SET_SEGV_LOCATION();
            denora_free(c->version);
        }
        denora_free(c);
        return 1;
    }
    return 0;
}

void ctcp_update(char *version)
{
    CTCPVerStats *c;

    SET_SEGV_LOCATION();

    c = find_ctcpver(version);
    if (!c) {
        return;
    }
    c->count--;
    version = rdb_escape(version);
    rdb_query("UPDATE %s SET count=%d, overall=%d WHERE version=\'%s\'",
              CTCPTable, c->count, c->overall, version);
    SET_SEGV_LOCATION();
    denora_free(version);
}

/*************************************************************************/

void handle_ctcp_version(char *nick, char *version)
{
    CTCPVerStats *c;
    User *u;

    SET_SEGV_LOCATION();

    if (LargeNet) {
        return;
    }

    if (!version) {
        /* client sent us nothing */
        return;
    }

    c = find_ctcpver(version);
    if (c) {
        c->count++;
        if (c->count > c->overall) {
            c->overall = c->count;
        }
        if (denora->do_sql) {
            sql_do_ctcp(UPDATE, version, c->count, c->overall);
        }
    } else {
        c = makectcp(version);
        c->count++;
        c->overall++;
        if (denora->do_sql) {
            sql_do_ctcp(ADD, version, 1, 1);
        }
    }
    u = finduser(nick);
    SET_SEGV_LOCATION();

    send_event(EVENT_CTCP_VERSION, 2, nick, version);

    if (u) {
        if (!u->ctcp) {
            u->ctcp = sstrdup(version);
        } else {
            denora_free(u->ctcp);
            u->ctcp = sstrdup(version);
        }
        version = rdb_escape(version);
        rdb_query
            ("UPDATE %s SET ctcpversion=\'%s\' WHERE nickid=%d",
             UserTable, version, db_getnick(u->nick));
        denora_free(version);
    }
}

/*************************************************************************/

void sql_do_ctcp(int type, char *version, int count, int overall)
{
    char *temp;
#ifdef USE_MYSQL
    MYSQL_RES *mysql_res;
#endif

    if (version) {
        temp = rdb_escape(version);

        if (type == 1) {
            SET_SEGV_LOCATION();
            rdb_query
                ("INSERT INTO %s (version, count, overall) VALUES(\'%s\', %d, %d)",
                 CTCPTable, temp, count, overall);
        }
        if (type == 4) {
            SET_SEGV_LOCATION();
            rdb_query("SELECT * FROM %s WHERE version=\'%s\'", CTCPTable,
                      temp);
#ifdef USE_MYSQL
            mysql_res = mysql_store_result(mysql);
            if (mysql_res) {
                if (mysql_num_rows(mysql_res)) {
                    rdb_query
                        ("UPDATE %s SET count=%d, overall=%d WHERE version=\'%s\'",
                         CTCPTable, count, overall, temp);
                } else {
                    rdb_query
                        ("INSERT INTO %s (version, count, overall) VALUES(\'%s\', %d, %d)",
                         CTCPTable, temp, count, overall);
                }
                SET_SEGV_LOCATION();
                mysql_free_result(mysql_res);
            }
#endif
        }
        SET_SEGV_LOCATION();
        denora_free(temp);
    }
}

/*************************************************************************/

CTCPVerStats *first_ctcp(void)
{
    next_index = 0;

    SET_SEGV_LOCATION();

    while (next_index < 1024 && current == NULL) {
        current = ctcplists[next_index++];
    }
    SET_SEGV_LOCATION();

    alog(LOG_EXTRADEBUG, "debug: first_ctcp() returning %s",
         current ? current->version : "NULL (end of list)");
    return current;
}

/*************************************************************************/

CTCPVerStats *next_ctcp(void)
{
    SET_SEGV_LOCATION();

    if (current)
        current = current->next;
    if (!current && next_index < 1024) {
        while (next_index < 1024 && current == NULL) {
            current = ctcplists[next_index++];
        }
    }
    SET_SEGV_LOCATION();

    alog(LOG_EXTRADEBUG, "debug: next_ctcp() returning %s",
         current ? current->version : "NULL (end of list)");
    return current;
}

/*************************************************************************/
