/* Hybrid IRCD functions
 *
 * (C) 2004-2005 Denora Team
 * Contact us at info@nomadirc.net
 *
 * Please read COPYING and README for furhter details.
 *
 * Based on the original code of Anope by Anope Team.
 * Based on the original code of Thales by Lucas.
 *
 * $Id: hybrid.c 433 2005-07-26 16:58:24Z drstein $
 *
 */

#include "denora.h"
#include "hybrid.h"

int ts6nickcount = 0;

IRCDVar myIrcd[] = {
    {"HybridIRCd 7.*",          /* ircd name */
     "+o",                      /* operserv mode */
     0,                         /* Vhost  */
     0,                         /* Supports SGlines     */
     0,                         /* sgline sql table */
     0,                         /* Supports SQlines     */
     0,                         /* sqline sql table */
     0,                         /* Supports SZlines     */
     1,                         /* Has exceptions +e    */
     0,                         /* vidents              */
     1,                         /* NICKIP               */
     0,                         /* VHOST ON NICK        */
     0,                         /* +f                   */
     0,                         /* +L                   */
     0,                         /* +f Mode                          */
     0,                         /* +L Mode                          */
     NULL,                      /* CAPAB Chan Modes             */
     0,                         /* We support TOKENS */
     1,                         /* TOKENS are CASE Sensitive */
     0,                         /* TIME STAMPS are BASE64 */
     1,                         /* +I support */
     0,                         /* SJOIN ban char */
     0,                         /* SJOIN except char */
     0,                         /* umode for vhost */
     0,                         /* owner */
     0,                         /* protect */
     1,                         /* halfop */
     "abcdfgiklnorsuwxyz",
     "beailmnpstkI",
     0,                         /* flood */
     0,                         /* flood other */
     0,                         /* vhost */
     0,                         /* vhost other */
     0,                         /* p10 */
     1,                         /* TS6 */
     1,
     0,
     0,
     'b',
     'e',
     'I',
     0,
     1,
     NULL,
     0},
};

IRCDCAPAB myIrcdcap[] = {
    {
     CAPAB_NOQUIT,              /* NOQUIT       */
     0,                         /* TSMODE       */
     0,                         /* UNCONNECT    */
     0,                         /* NICKIP       */
     0,                         /* SJOIN        */
     CAPAB_ZIP,                 /* ZIP          */
     0,                         /* BURST        */
     CAPAB_TS5,                 /* TS5          */
     0,                         /* TS3          */
     0,                         /* DKEY         */
     0,                         /* PT4          */
     0,                         /* SCS          */
     CAPAB_QS,                  /* QS           */
     CAPAB_UID,                 /* UID          */
     CAPAB_KNOCK,               /* KNOCK        */
     0,                         /* CLIENT       */
     0,                         /* IPV6         */
     0,                         /* SSJ5         */
     0,                         /* SN2          */
     0,                         /* TOKEN        */
     0,                         /* VHOST        */
     0,                         /* SSJ3         */
     0,                         /* NICK2        */
     0,                         /* UMODE2       */
     0,                         /* VL           */
     0,                         /* TLKEXT       */
     0,                         /* DODKEY       */
     0,                         /* DOZIP        */
     0, 0, 0}
};

unsigned long myumodes[128] = {
    0, 0, 0,                    /* Unused */
    0, 0, 0,                    /* Unused */
    0, 0, 0,                    /* Unused, Unused, Horzontal Tab */
    0, 0, 0,                    /* Line Feed, Unused, Unused */
    0, 0, 0,                    /* Carriage Return, Unused, Unused */
    0, 0, 0,                    /* Unused */
    0, 0, 0,                    /* Unused */
    0, 0, 0,                    /* Unused */
    0, 0, 0,                    /* Unused */
    0, 0, 0,                    /* Unused */
    0, 0, 0,                    /* Unused, Unused, Space */
    0, 0, 0,                    /* ! " #  */
    0, 0, 0,                    /* $ % &  */
    0, 0, 0,                    /* ! ( )  */
    0, 0, 0,                    /* * + ,  */
    0, 0, 0,                    /* - . /  */
    0, 0,                       /* 0 1 */
    0, 0,                       /* 2 3 */
    0, 0,                       /* 4 5 */
    0, 0,                       /* 6 7 */
    0, 0,                       /* 8 9 */
    0, 0,                       /* : ; */
    0, 0, 0,                    /* < = > */
    0, 0,                       /* ? @ */
    0, 0, 0,                    /* A B C */
    0, 0, 0,                    /* D E F */
    0, 0, 0,                    /* G H I */
    0, 0, 0,                    /* J K L */
    0, 0, 0,                    /* M N O */
    0, 0, 0,                    /* P Q R */
    0, 0, 0,                    /* S T U */
    0, 0, 0,                    /* V W X */
    0,                          /* Y */
    0,                          /* Z */
    0, 0, 0,                    /* [ \ ] */
    0, 0, 0,                    /* ^ _ ` */
    IRC_UMODE_a, IRC_UMODE_b, IRC_UMODE_c,  /* a b c */
    IRC_UMODE_d, 0, IRC_UMODE_f,        /* d e f */
    IRC_UMODE_g, 0, IRC_UMODE_i,        /* g h i */
    0, IRC_UMODE_k, IRC_UMODE_l,        /* j k l */
    0, IRC_UMODE_n, IRC_UMODE_o,        /* m n o */
    0, 0, IRC_UMODE_r,              /* p q r */
    IRC_UMODE_s, 0, IRC_UMODE_u,        /* s t u */
    0, IRC_UMODE_w, IRC_UMODE_x,        /* v w x */
    IRC_UMODE_y,                    /* y */
    IRC_UMODE_z,                    /* z */
    0, 0, 0,                    /* { | } */
    0, 0                        /* ~  */
};


char myCsmodes[128] = {
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,

    0,
    'a',                        /* (33) !  */
    0, 0, 0,
    'h',
    0, 0, 0, 0,
    0,

    'v', 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,

    'o', 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,

    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

CMMode myCmmodes[128] = {
    {NULL, NULL}, {NULL, NULL}, {NULL, NULL}, {NULL, NULL}, {NULL, NULL}, {NULL, NULL}, {NULL, NULL}, {NULL, NULL},
    {NULL, NULL}, {NULL, NULL}, {NULL, NULL}, {NULL, NULL}, {NULL, NULL}, {NULL, NULL}, {NULL, NULL}, {NULL, NULL},
    {NULL, NULL}, {NULL, NULL}, {NULL, NULL}, {NULL, NULL}, {NULL, NULL}, {NULL, NULL}, {NULL, NULL}, {NULL, NULL},
    {NULL, NULL}, {NULL, NULL}, {NULL, NULL}, {NULL, NULL}, {NULL, NULL}, {NULL, NULL}, {NULL, NULL}, {NULL, NULL},
    {NULL, NULL}, {NULL, NULL}, {NULL, NULL}, {NULL, NULL}, {NULL, NULL}, {NULL, NULL}, {NULL, NULL}, {NULL, NULL},
    {NULL, NULL}, {NULL, NULL}, {NULL, NULL}, {NULL, NULL}, {NULL, NULL}, {NULL, NULL}, {NULL, NULL}, {NULL, NULL},
    {NULL, NULL}, {NULL, NULL}, {NULL, NULL}, {NULL, NULL}, {NULL, NULL}, {NULL, NULL}, {NULL, NULL}, {NULL, NULL},
    {NULL, NULL}, {NULL, NULL}, {NULL, NULL}, {NULL, NULL}, {NULL, NULL}, {NULL, NULL}, {NULL, NULL}, {NULL, NULL},
    {NULL, NULL}, 
    {NULL, NULL},                     /* A */
    {NULL, NULL}, {NULL, NULL}, {NULL, NULL},     /* BCD */
    {NULL, NULL}, {NULL, NULL}, {NULL, NULL},     /* EFG */
    {NULL, NULL},                     /* H */
    {add_invite, del_invite},   /* I */
    {NULL, NULL},                     /* J */
    {NULL, NULL}, {NULL, NULL}, {NULL, NULL},     /* KLM */
    {NULL, NULL}, {NULL, NULL}, {NULL, NULL},     /* NOP */
    {NULL, NULL}, {NULL, NULL}, {NULL, NULL},     /* QRS */
    {NULL, NULL}, {NULL, NULL}, {NULL, NULL},     /* TUV */
    {NULL, NULL}, {NULL, NULL}, {NULL, NULL},     /* WXY */
    {NULL, NULL},                     /* Z */
    {NULL, NULL}, {NULL, NULL},             /* (char 91 - 92) */
    {NULL, NULL}, {NULL, NULL}, {NULL, NULL},     /* (char 93 - 95) */
    {NULL, NULL},                     /* ` (char 96) */
    {NULL, NULL},                     /* a (char 97) */
    {add_ban, del_ban},         /* b */
    {NULL, NULL}, {NULL, NULL},             /* cd */
    {add_exception, del_exception},
    {NULL, NULL}, {NULL, NULL},
    {NULL, NULL}, {NULL, NULL}, {NULL, NULL}, {NULL, NULL}, {NULL, NULL}, {NULL, NULL}, {NULL, NULL}, {NULL, NULL},
    {NULL, NULL}, {NULL, NULL}, {NULL, NULL}, {NULL, NULL}, {NULL, NULL}, {NULL, NULL}, {NULL, NULL}, {NULL, NULL},
    {NULL, NULL}, {NULL, NULL}, {NULL, NULL}, {NULL, NULL}, {NULL, NULL}, {NULL, NULL}, {NULL, NULL}, {NULL, NULL},
};


CBMode myCbmodes[128] = {
    {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0},
    {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0},
    {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0},
    {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0},
    {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0},
    {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0},
    {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0},
    {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0},
    {0, 0},
    {0, 0},                        /* A */
    {0, 0},                        /* B */
    {0, 0},                        /* C */
    {0, 0},                        /* D */
    {0, 0},                        /* E */
    {0, 0},                        /* F */
    {0, 0},                        /* G */
    {0, 0},                        /* H */
    {0, 0},                        /* I */
    {0, 0},                        /* J */
    {0, 0},                        /* K */
    {0, 0},                        /* L */
    {0, 0},                        /* M */
    {0, 0},                        /* N */
    {0, 0},                        /* O */
    {0, 0},                        /* P */
    {0, 0},                        /* Q */
    {0, 0},
    {0, 0},                        /* S */
    {0, 0},                        /* T */
    {0, 0},                        /* U */
    {0, 0},                        /* V */
    {0, 0},                        /* W */
    {0, 0},                        /* X */
    {0, 0},                        /* Y */
    {0, 0},                        /* Z */
    {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0},
    {IRCD_CMODE_a, NULL},            /* a */
    {0, 0},                        /* b */
    {0, 0},                        /* c */
    {0, 0},                        /* d */
    {0, 0},                        /* e */
    {0, 0},                        /* f */
    {0, 0},                        /* g */
    {0, 0},                        /* h */
    {IRCD_CMODE_i, NULL},            /* i */
    {0, 0},                        /* j */
    {IRCD_CMODE_k, set_key},         /* k */
    {IRCD_CMODE_l, set_limit},       /* l */
    {IRCD_CMODE_m, NULL},            /* m */
    {IRCD_CMODE_n, NULL},            /* n */
    {0, 0},                        /* o */
    {IRCD_CMODE_p, NULL},            /* p */
    {0, 0},                        /* q */
    {0, 0},                        /* r */
    {IRCD_CMODE_s, NULL},            /* s */
    {IRCD_CMODE_t, NULL},            /* t */
    {0, 0},                        /* u */
    {0, 0},                        /* v */
    {0, 0},                        /* w */
    {0, 0},                        /* x */
    {0, 0},                        /* y */
    {0, 0},                        /* z */
    {0, 0}, {0, 0}, {0, 0}, {0, 0}

};

CBModeInfo myCbmodeinfos[] = {
    {'a', IRCD_CMODE_a, NULL},
    {'i', IRCD_CMODE_i, NULL},
    {'k', IRCD_CMODE_k, get_key},
    {'l', IRCD_CMODE_l, get_limit},
    {'m', IRCD_CMODE_m, NULL},
    {'n', IRCD_CMODE_n, NULL},
    {'p', IRCD_CMODE_p, NULL},
    {'s', IRCD_CMODE_s, NULL},
    {'t', IRCD_CMODE_t, NULL},
};

void hybrid_cmd_notice(char *source, char *dest, char *buf)
{
    Uid *ud;
    User *u;

    ud = find_uid(source);
    u = finduser(dest);
    send_cmd((UseTS6 ? (ud ? ud->uid : source) : source),
                 "NOTICE %s :%s", (UseTS6 ? (u ? u->uid : dest) : dest),
                 buf);
}

void hybrid_cmd_privmsg(char *source, char *dest, char *buf)
{
    Uid *ud, *ud2;

    ud = find_uid(source);
    ud2 = find_uid(dest);

    send_cmd((UseTS6 ? (ud ? ud->uid : source) : source), "PRIVMSG %s :%s",
             (UseTS6 ? (ud2 ? ud2->uid : dest) : dest), buf);
}

void hybrid_cmd_serv_notice(char *source, char *dest, char *msg)
{
    send_cmd(source, "NOTICE $$%s :%s", dest, msg);
}

void hybrid_cmd_serv_privmsg(char *source, char *dest, char *msg)
{
    send_cmd(source, "PRIVMSG $$%s :%s", dest, msg);
}

void hybrid_cmd_global(char *source, char *buf)
{
    Uid *u;

    if (source) {
        u = find_uid(source);
        if (u) {
            send_cmd((UseTS6 ? u->uid : source), "OPERWALL :%s", buf);
        } else {
            send_cmd((UseTS6 ? TS6SID : ServerName), "OPERWALL :%s", buf);
        }
    } else {
        send_cmd((UseTS6 ? TS6SID : ServerName), "OPERWALL :%s", buf);
    }
}

int denora_event_sjoin(char *source, int ac, char **av)
{
  if (denora->protocoldebug) {
        protocol_debug(source, ac, av);
  }
    do_sjoin(source, ac, av);
    return MOD_CONT;
}

/*
  1 : const char *source
  2 : char *nick
  3 : char *username
  4 : char *host,
  5 : char *server
  6 : char *realname
  7 : time_t ts
  8 : uint32 svid
  9 : uint32 ip
 10 : char *vhost 
 11 : char *uid
 12 : int hopcount
 13 : char *modes
*/
int denora_event_nick(char *source, int ac, char **av)
{
    Server *s;
    User *user;

  if (denora->protocoldebug) {
        protocol_debug(source, ac, av);
  }

    if (UseTS6) {
      if (ac == 9) {
        s = findserver_uid(source);
        /* Source is always the server */
        *source = '\0';
        user = do_nick(source, av[0], av[4], av[5], s->name, av[8],
                       strtoul(av[2], NULL, 10), 0, 0, NULL, av[7],
                       strtoul(av[1], NULL, 10), av[3]);
        if (user) {
            denora_set_umode(user, 1, &av[3]);
        }
       } else {
            user = find_byuid(source);
            do_nick((user ? user->nick : source), av[0], NULL, NULL, NULL, NULL,
                    strtoul(av[1], NULL, 10), 0, 0, NULL, NULL, 0, NULL);
       }
    } else {
        if (ac != 2) {
            user = do_nick(source, av[0], av[4], av[5], av[6], av[7],
                           strtoul(av[2], NULL, 10), 0, 0, NULL, NULL,
                           strtoul(av[1], NULL, 10), av[3]);
            if (user)
                denora_set_umode(user, 1, &av[3]);
        } else {
            do_nick(source, av[0], NULL, NULL, NULL, NULL,
                    strtoul(av[1], NULL, 10), 0, 0, NULL, NULL, 0, NULL);
        }
    }
    return MOD_CONT;
}

int denora_event_topic(char *source, int ac, char **av)
{
  if (denora->protocoldebug) {
        protocol_debug(source, ac, av);
  }

    do_topic(ac, av);
    return MOD_CONT;
}

int denora_event_tburst(char *source, int ac, char **av)
{
  if (denora->protocoldebug) {
        protocol_debug(source, ac, av);
  }
    if (ac != 5)
        return MOD_CONT;

    av[0] = av[1];
    av[1] = av[3];
    av[3] = av[4];
    do_topic(4, av);
    return MOD_CONT;
}

int denora_event_436(char *source, int ac, char **av)
{
  if (denora->protocoldebug) {
        protocol_debug(source, ac, av);
  }
    if (ac < 1)
        return MOD_CONT;

    m_nickcoll(av[0]);
    return MOD_CONT;
}

int denora_event_pong(char *source, int ac, char **av)
{
  if (denora->protocoldebug) {
        protocol_debug(source, ac, av);
  }
  server_store_pong(source, time(NULL));
  return MOD_CONT;
}

/* *INDENT-OFF* */
void moduleAddIRCDMsgs(void) {
    Message *m;

    if (UseTS6) {
        if (!BadPtr(Numeric)) {
            TS6SID = sstrdup(Numeric);
            UseTSMODE = 1;  /* TMODE */
        } else {
            alog(LOG_NORMAL, "TS6 requires the use of Numeric - exiting");
            exit(1);
        }
    }

    m = createMessage("436",       denora_event_436); addCoreMessage(IRCD,m);
    m = createMessage("AWAY",      denora_event_away); addCoreMessage(IRCD,m);
    m = createMessage("INVITE",    denora_event_null); addCoreMessage(IRCD,m);
    m = createMessage("JOIN",      denora_event_join); addCoreMessage(IRCD,m);
    m = createMessage("KICK",      denora_event_kick); addCoreMessage(IRCD,m);
    m = createMessage("KILL",      denora_event_kill); addCoreMessage(IRCD,m);
    m = createMessage("MODE",      denora_event_mode); addCoreMessage(IRCD,m);
    m = createMessage("PONG",      denora_event_pong); addCoreMessage(IRCD,m);
    m = createMessage("MOTD",      denora_event_motd); addCoreMessage(IRCD,m);
    m = createMessage("NICK",      denora_event_nick); addCoreMessage(IRCD,m);
    m = createMessage("NOTICE",    denora_event_notice); addCoreMessage(IRCD,m);
    m = createMessage("PART",      denora_event_part); addCoreMessage(IRCD,m);
    m = createMessage("PASS",      denora_event_pass); addCoreMessage(IRCD,m);
    m = createMessage("PING",      denora_event_ping); addCoreMessage(IRCD,m);
    m = createMessage("PRIVMSG",   denora_event_privmsg); addCoreMessage(IRCD,m);
    m = createMessage("QUIT",      denora_event_quit); addCoreMessage(IRCD,m);
    m = createMessage("SERVER",    denora_event_server); addCoreMessage(IRCD,m);
    m = createMessage("SQUIT",     denora_event_squit); addCoreMessage(IRCD,m);
    m = createMessage("TOPIC",     denora_event_topic); addCoreMessage(IRCD,m);
    m = createMessage("USER",      denora_event_null); addCoreMessage(IRCD,m);
    m = createMessage("WALLOPS",   denora_event_null); addCoreMessage(IRCD,m);
    m = createMessage("WHOIS",     denora_event_whois); addCoreMessage(IRCD,m);
    m = createMessage("AKILL",     denora_event_null); addCoreMessage(IRCD,m);
    m = createMessage("GLOBOPS",   denora_event_null); addCoreMessage(IRCD,m);
    m = createMessage("GNOTICE",   denora_event_null); addCoreMessage(IRCD,m);
    m = createMessage("GOPER",     denora_event_null); addCoreMessage(IRCD,m);
    m = createMessage("RAKILL",    denora_event_null); addCoreMessage(IRCD,m);
    m = createMessage("SILENCE",   denora_event_null); addCoreMessage(IRCD,m);
    m = createMessage("SVSKILL",   denora_event_null); addCoreMessage(IRCD,m);
    m = createMessage("SVSMODE",   denora_event_null); addCoreMessage(IRCD,m);
    m = createMessage("SVSNICK",   denora_event_null); addCoreMessage(IRCD,m);
    m = createMessage("SVSNOOP",   denora_event_null); addCoreMessage(IRCD,m);
    m = createMessage("SQLINE",    denora_event_null); addCoreMessage(IRCD,m);
    m = createMessage("UNSQLINE",  denora_event_null); addCoreMessage(IRCD,m);
    m = createMessage("CAPAB",     denora_event_capab); addCoreMessage(IRCD,m);
    m = createMessage("SJOIN",     denora_event_sjoin); addCoreMessage(IRCD,m);
    m = createMessage("SVINFO",    denora_event_svinfo); addCoreMessage(IRCD,m);
    m = createMessage("EOB",       denora_event_eob); addCoreMessage(IRCD,m);
    m = createMessage("ADMIN",     denora_event_null); addCoreMessage(IRCD,m);
    m = createMessage("ERROR",     denora_event_error); addCoreMessage(IRCD,m);
    m = createMessage("SID",       denora_event_sid); addCoreMessage(IRCD,m);
    m = createMessage("BMASK",     denora_event_bmask); addCoreMessage(IRCD,m);
    m = createMessage("UID",       denora_event_nick); addCoreMessage(IRCD,m);
    m = createMessage("TMODE",     denora_event_tmode); addCoreMessage(IRCD,m);
}

/* *INDENT-ON* */

void hybrid_cmd_join(char *user, char *channel, time_t chantime)
{
    Uid *ud;

    ud = find_uid(user);
    send_cmd(NULL, "SJOIN %ld %s + :@%s", (long int) chantime,
             channel, (UseTS6 ? (ud ? ud->uid : user) : user));

    if (AutoOp)
        send_cmd((UseTS6 ? TS6SID : ServerName), "MODE %s +o %s", channel, (UseTS6 ? (ud ? ud->uid : user) : user));
}

void hybrid_cmd_version(char *server)
{
    Uid *ud;
    ud = find_uid(s_StatServ);

    send_cmd((ud ? ud->uid : s_StatServ), "VERSION %s", server);
}

void hybrid_cmd_motd(char *sender, char *server)
{
    Uid *ud;
    ud = find_uid(sender);

    send_cmd((ud ? ud->uid : sender), "MOTD %s", server);
}

/*
 * SVINFO
 *      parv[0] = sender prefix
 *      parv[1] = TS_CURRENT for the server
 *      parv[2] = TS_MIN for the server
 *      parv[3] = server is standalone or connected to non-TS only
 *      parv[4] = server's idea of UTC time
 */
void hybrid_cmd_svinfo(void)
{
    if (UseTS6) {
        send_cmd(NULL, "SVINFO 6 3 0 :%ld", (long int) time(NULL));
    } else {
        send_cmd(NULL, "SVINFO 5 5 0 :%ld", (long int) time(NULL));
    }
}

/* CAPAB */
/*
  QS     - Can handle quit storm removal
  EX     - Can do channel +e exemptions 
  CHW    - Can do channel wall @#
  LL     - Can do lazy links 
  IE     - Can do invite exceptions 
  EOB    - Can do EOB message
  KLN    - Can do KLINE message 
  GLN    - Can do GLINE message 
  HOPS   - can do half ops (+h)
  HUB    - This server is a HUB 
  AOPS   - Can do anon ops (+a) 
  UID    - Can do UIDs
  ZIP    - Can do ZIPlinks
  ENC    - Can do ENCrypted links 
  KNOCK  -  supports KNOCK 
  TBURST - supports TBURST
  PARA	 - supports invite broadcasting for +p
  ENCAP	 - ?
*/
void hybrid_cmd_capab(void)
{
    send_cmd(NULL,
             "CAPAB :QS EX CHW IE EOB KLN GLN HOPS HUB AOPS KNOCK TBURST PARA");
}

void hybrid_cmd_stats(char *sender, const char *letter, char *server)
{
    Uid *ud;
    ud = find_uid(sender);

    send_cmd((ud ? ud->uid : sender), "STATS %s %s", letter, server);
}

/* PASS */
void hybrid_cmd_pass(char *pass)
{
    if (UseTS6) {
        send_cmd(NULL, "PASS %s TS 6 :%s", pass, TS6SID);
    } else {
        send_cmd(NULL, "PASS %s :TS", pass);
    }
}

/* SERVER name hop descript */
void hybrid_cmd_server(char *servname, int hop, char *descript)
{
    send_cmd(NULL, "SERVER %s %d :%s", servname, hop, descript);
}
void hybrid_cmd_connect()
{
    char *hops;

    hops = sstrdup("0");
    /* Make myself known to myself in the serverlist */
    if (UseTS6 && Numeric) {
        me_server = do_server(NULL, ServerName, hops, ServerDesc, TS6SID);
    } else {
        me_server = do_server(NULL, ServerName, hops, ServerDesc, NULL);
    }
    denora_free(hops);

    hybrid_cmd_pass(RemotePassword);
    hybrid_cmd_capab();
    hybrid_cmd_server(ServerName, 1, ServerDesc);
    hybrid_cmd_svinfo();
}

void hybrid_cmd_bot_nick(char *nick, char *user, char *host, char *real,
                        char *modes)
{
    char nicknumbuf[10];
    if (UseTS6) {
        snprintf(nicknumbuf, 10, "%sAAAAA%c", Numeric,
                 (ts6nickcount + 'A'));
        send_cmd(TS6SID, "UID %s 1 %ld %s %s %s 0 %s :%s", nick,
                 (long int) time(NULL), modes, user, host, nicknumbuf,
                 real);
        new_uid(nick, nicknumbuf);
        ts6nickcount++;
    } else {
        send_cmd(NULL, "NICK %s 1 %ld %s %s %s %s :%s", nick,
                 (long int) time(NULL), modes, user, host, ServerName,
                 real);
    }
}

void hybrid_cmd_part(char *nick, char *chan, char *buf)
{
    Uid *ud;

    ud = find_uid(nick);

    if (buf) {
        send_cmd((UseTS6 ? ud->uid : nick), "PART %s :%s", chan, buf);
    } else {
        send_cmd((UseTS6 ? ud->uid : nick), "PART %s", chan);
    }
}

int denora_event_ping(char *source, int ac, char **av)
{
  if (denora->protocoldebug) {
        protocol_debug(source, ac, av);
  }
    if (ac < 1)
        return MOD_CONT;
    hybrid_cmd_pong(ac > 1 ? av[1] : ServerName, av[0]);
    return MOD_CONT;
}

int denora_event_away(char *source, int ac, char **av)
{
    User *u = NULL;

  if (denora->protocoldebug) {
        protocol_debug(source, ac, av);
  }

    if (UseTS6) {
        u = find_byuid(source);
    }

    if (!ac) {
       m_away((UseTS6 ? (u ? u->nick : source) : source), NULL);
    } else {
       m_away((UseTS6 ? (u ? u->nick : source) : source), av[0]);
    }
    return MOD_CONT;
}

int denora_event_kill(char *source, int ac, char **av)
{
  if (denora->protocoldebug) {
        protocol_debug(source, ac, av);
  }
    if (ac != 2)
        return MOD_CONT;

    m_kill(source, av[0], av[1]);
    return MOD_CONT;
}

int denora_event_kick(char *source, int ac, char **av)
{
  if (denora->protocoldebug) {
        protocol_debug(source, ac, av);
  }
    if (ac != 3)
        return MOD_CONT;
    do_kick(source, ac, av);
    return MOD_CONT;
}

int denora_event_eob(char *source, int ac, char **av)
{
  if (denora->protocoldebug) {
        protocol_debug(source, ac, av);
  }
    update_sync_state(source, 1);
    return MOD_CONT;
}

void hybrid_cmd_eob()
{
    send_cmd(ServerName, "EOB");
}

void hybrid_cmd_ping(char *server)
{
    send_cmd(ServerName, "PING %s :%s", ServerName, server);
}

int denora_event_join(char *source, int ac, char **av)
{
  if (denora->protocoldebug) {
        protocol_debug(source, ac, av);
  }
    if (ac != 1)
        return MOD_CONT;
    do_join(source, ac, av);
    return MOD_CONT;
}

int denora_event_motd(char *source, int ac, char **av)
{
  if (denora->protocoldebug) {
        protocol_debug(source, ac, av);
  }
  m_motd(source);
  return MOD_CONT;
}

int denora_event_privmsg(char *source, int ac, char **av)
{
    User *u = NULL;
    Uid *ud = NULL;

  if (denora->protocoldebug) {
        protocol_debug(source, ac, av);
  }
    if (ac != 2) {
        return MOD_CONT;
    }

    if (UseTS6) {
        u = find_byuid(source);
        ud = find_nickuid(av[0]);
    }
    m_privmsg((UseTS6 ? (u ? u->nick : source) : source),
              (UseTS6 ? (ud ? ud->nick : av[0]) : av[0]), av[1]);
    return MOD_CONT;
}

int denora_event_part(char *source, int ac, char **av)
{
    User *u;

  if (denora->protocoldebug) {
        protocol_debug(source, ac, av);
  }

    if (ac < 1 || ac > 2) {
        return MOD_CONT;
    }

    u = find_byuid(source);
    do_part((UseTS6 ? (u ? u->nick : source) : source), ac, av);

    return MOD_CONT;
}

int denora_event_whois(char *source, int ac, char **av)
{
    Uid *ud;

  if (denora->protocoldebug) {
        protocol_debug(source, ac, av);
  }

    if (source && ac >= 1) {
        ud = find_nickuid(av[0]);
        m_whois(source, (UseTS6 ? (ud ? ud->nick : av[0]) : av[0]));
    }
    return MOD_CONT;
}

/* EVENT: SERVER */
int denora_event_server(char *source, int ac, char **av)
{
  if (denora->protocoldebug) {
        protocol_debug(source, ac, av);
  }
    if (!stricmp(av[1], "1")) {
        denora->uplink = sstrdup(av[0]);
        me_server->uplink = sstrdup(av[0]);
        if (UseTS6 && TS6UPLINK) {
            do_server(source, av[0], av[1], av[2], TS6UPLINK);
        } else {
            do_server(source, av[0], av[1], av[2], NULL);
        }
    } else {
        do_server(source, av[0], av[1], av[2], NULL);
    }
    return MOD_CONT;
}

int denora_event_sid(char *source, int ac, char **av)
{
    Server *s;

    /* :42X SID trystan.nomadirc.net 2 43X :ircd-ratbox test server */

  if (denora->protocoldebug) {
        protocol_debug(source, ac, av);
  }

    s = findserver_uid(source);

    do_server(s->name, av[0], av[1], av[3], av[2]);
    return MOD_CONT;
}

int denora_event_squit(char *source, int ac, char **av)
{
  if (denora->protocoldebug) {
        protocol_debug(source, ac, av);
  }
    if (ac != 2)
        return MOD_CONT;
    do_squit(av[0]);
    return MOD_CONT;
}

int denora_event_quit(char *source, int ac, char **av)
{
  if (denora->protocoldebug) {
        protocol_debug(source, ac, av);
  }
    if (ac != 1)
        return MOD_CONT;
    do_quit(source, ac, av);
    return MOD_CONT;
}

void hybrid_cmd_mode(char *source, char *dest, char *buf)
{
    Uid *ud;

    if (source) {
        ud = find_uid(source);
        send_cmd((UseTS6 ? (ud ? ud->uid : source) : source), "MODE %s %s",
                 dest, buf);
    } else {
        send_cmd(source, "MODE %s %s", dest, buf);
    }
}

void hybrid_cmd_nick(char *nick, char *name, const char *mode)
{
    char nicknumbuf[10];
    if (UseTS6) {
        snprintf(nicknumbuf, 10, "%sAAAAA%c", Numeric,
                 (ts6nickcount + 'A'));
        send_cmd(TS6SID, "UID %s 1 %ld %s %s %s 0 %s :%s", nick,
                 (long int) time(NULL), mode, ServiceUser, ServiceHost,
                 nicknumbuf, name);
        new_uid(nick, nicknumbuf);
        ts6nickcount++;
    } else {
        send_cmd(NULL, "NICK %s 1 %ld %s %s %s %s :%s", nick,
                 (long int) time(NULL), mode, ServiceUser, ServiceHost,
                 ServerName, name);
    }
}

/* QUIT */
void hybrid_cmd_quit(char *source, char *buf)
{
    Uid *ud;
    ud = find_uid(source);

    if (buf) {
        send_cmd((UseTS6 ? (ud ? ud->uid : source) : source), "QUIT :%s",
                 buf);
    } else {
        send_cmd((UseTS6 ? (ud ? ud->uid : source) : source), "QUIT");
    }
}

int denora_event_pass(char *source, int ac, char **av)
{
  if (denora->protocoldebug) {
        protocol_debug(source, ac, av);
  }
    if (UseTS6) {
        TS6UPLINK = sstrdup(av[3]);
    }
    return MOD_CONT;
}

/* :42X BMASK 1106409026 #ircops b :*!*@*.aol.com */
int denora_event_bmask(char *source, int ac, char **av)
{
  if (denora->protocoldebug) {
        protocol_debug(source, ac, av);
  }
    do_bmask(av);
    return MOD_CONT;
}

/* PONG */
void hybrid_cmd_pong(char *servname, char *who)
{
    if (UseTS6) {
        send_cmd(TS6SID, "PONG %s", who);
    } else {
        send_cmd(servname, "PONG %s", who);
    }
}

/* SQUIT */
void hybrid_cmd_squit(char *servname, char *message)
{
    send_cmd(NULL, "SQUIT %s :%s", servname, message);
}

int denora_event_mode(char *source, int ac, char **av)
{
    User *u, *u2;

  if (denora->protocoldebug) {
        protocol_debug(source, ac, av);
  }

    if (ac < 2) {
        return MOD_CONT;
    }

    if (*av[0] == '#' || *av[0] == '&') {
        do_cmode(source, ac, av);
    } else {
        if (UseTS6) {
            u = find_byuid(source);
            u2 = find_byuid(av[0]);
            av[0] = u2->nick;
            do_umode(u->nick, ac, av);
        } else {
            do_umode(source, ac, av);
        }
    }
    return MOD_CONT;
}

int denora_event_tmode(char *source, int ac, char **av)
{
  if (denora->protocoldebug) {
        protocol_debug(source, ac, av);
  }
    if (*av[1] == '#' || *av[1] == '&') {
        do_cmode(source, ac, av);
    }
    return MOD_CONT;
}

void hybrid_cmd_351(char *source)
{
    send_cmd(ServerName,
             "351 %s Denora-%s %s :%s -- build #%s, compiled %s %s",
             source, denora->version, ServerName, ircd->name, denora->build, denora->date, denora->time);
}

/* Event: PROTOCTL */
int denora_event_capab(char *source, int ac, char **av)
{
  if (denora->protocoldebug) {
        protocol_debug(source, ac, av);
  }
    capab_parse(ac, av);
    return MOD_CONT;
}

/*
 * SVINFO
 *      parv[0] = sender prefix
 *      parv[1] = TS_CURRENT for the server
 *      parv[2] = TS_MIN for the server
 *      parv[3] = server is standalone or connected to non-TS only
 *      parv[4] = server's idea of UTC time
 */
int denora_event_svinfo(char *source, int ac, char **av)
{
  if (denora->protocoldebug) {
        protocol_debug(source, ac, av);
  }
    /* currently not used but removes the message : unknown message from server */
    return MOD_CONT;
}

int denora_event_error(char *source, int ac, char **av)
{
  if (denora->protocoldebug) {
        protocol_debug(source, ac, av);
  }
    if (ac >= 1) {
        alog(LOG_ERROR, "ERROR: %s", av[0]);
    }
    return MOD_CONT;
}

void hybrid_cmd_ctcp(char *source, char *dest, char *buf)
{
    send_cmd(source, "NOTICE %s :\1%s \1", dest, buf);
}

int denora_event_notice(char *source, int ac, char **av)
{
  if (denora->protocoldebug) {
        protocol_debug(source, ac, av);
  }
    if (ac != 2) {
        return MOD_CONT;
    }
    m_notice(source, av[0], av[1]);
    return MOD_CONT;
}

void moduleAddIRCDCmds() {
    pmodule_cmd_nick(hybrid_cmd_nick);
    pmodule_cmd_mode(hybrid_cmd_mode);
    pmodule_cmd_bot_nick(hybrid_cmd_bot_nick);
    pmodule_cmd_notice(hybrid_cmd_notice);
    pmodule_cmd_privmsg(hybrid_cmd_privmsg);
    pmodule_cmd_serv_notice(hybrid_cmd_serv_notice);
    pmodule_cmd_serv_privmsg(hybrid_cmd_serv_privmsg);
    pmodule_cmd_351(hybrid_cmd_351);
    pmodule_cmd_quit(hybrid_cmd_quit);
    pmodule_cmd_pong(hybrid_cmd_pong);
    pmodule_cmd_join(hybrid_cmd_join);
    pmodule_cmd_part(hybrid_cmd_part);
    pmodule_cmd_global(hybrid_cmd_global);
    pmodule_cmd_squit(hybrid_cmd_squit);
    pmodule_cmd_connect(hybrid_cmd_connect);
    pmodule_cmd_eob(hybrid_cmd_eob);
    pmodule_cmd_ctcp(hybrid_cmd_ctcp);
    pmodule_cmd_version(hybrid_cmd_version);
    pmodule_cmd_stats(hybrid_cmd_stats);
    pmodule_cmd_motd(hybrid_cmd_motd);
    pmodule_cmd_ping(hybrid_cmd_ping);
}

int DenoraInit(int argc, char **argv) 
{
  if (denora->protocoldebug) {
        protocol_debug(NULL, argc, argv);
  }

	moduleAddAuthor("Denora");
    moduleAddVersion("$Id: hybrid.c 433 2005-07-26 16:58:24Z drstein $");
    moduleSetType(PROTOCOL);

    pmodule_ircd_version("Hybrid IRCd 7.0");
    pmodule_ircd_cap(myIrcdcap);
    pmodule_ircd_var(myIrcd);
    pmodule_ircd_cbmodeinfos(myCbmodeinfos);
    pmodule_ircd_cbmodes(myCbmodes);
    pmodule_ircd_cmmodes(myCmmodes);
    pmodule_ircd_csmodes(myCsmodes);
    pmodule_ircd_umodes(myumodes);
	pmodule_ircd_useTSMode(0);
	pmodule_oper_umode(IRC_UMODE_o);
    pmodule_irc_var(IRC_HYBRID);
    moduleAddIRCDCmds();
    moduleAddIRCDMsgs();

    return MOD_CONT;
}


